package com.uas.ps.inquiry.dao;

import com.uas.ps.inquiry.model.InquiryEnRemind;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;

import javax.transaction.Transactional;
import java.util.List;


/**
 * 询价企业推荐表数据库操作
 *
 * Created by liusw on 2018-06-04.
 */
@Repository
public interface InquiryEnRemindDao extends JpaRepository<InquiryEnRemind, Long>, JpaSpecificationExecutor<InquiryEnRemind> {

    /**
     * 通过vendUU和公共询价明细id更新该单据的状态
     *
     * @param sourceId 公共询价明细id
     * @param value 单据状态
     *              <note>
     *                  {@code 100 在录入}
     *                  {@code 101 已提交}
     *                  {@code 102 已审核}
     *                  {@code 103 未通过}
     *                  {@code 104 已通过}
     *              </note>
     * @param vendUU 被推荐的企业
     */
    @Transactional
    @Modifying
    @Query("update InquiryEnRemind set status = :value where itemId = :sourceId and vendUU = :vendUU")
    void updateStatus(@Param("sourceId") Long sourceId, @Param("value") int value, @Param("vendUU") Long vendUU);

    /**
     * 通过vendUU和公共询价明细id更新该单据的状态
     * @param id 报价明细id
     * @param sourceId 公共询价明细id
     * @param value 单据状态
     *              <note>
     *                  {@code 100 在录入}
     *                  {@code 101 已提交}
     *                  {@code 102 已审核}
     *                  {@code 103 未通过}
     *                  {@code 104 已通过}
     *              </note>
     * @param vendUU 被推荐的企业
     */
    @Transactional
    @Modifying
    @Query("update InquiryEnRemind set status = :value, newId = :id where itemId = :sourceId and vendUU = :vendUU")
    void updateStatus(@Param("id") Long id, @Param("sourceId") Long sourceId, @Param("value") int value, @Param("vendUU") Long vendUU);

    /**
     * 通过明细id和企业uu查询企业推荐信息
     * @param itemId
     * @param enUU
     * @return
     */
    InquiryEnRemind findByItemIdAndEnUU(Long itemId, Long enUU);

    /**
     * 通过明细id和供应商uu查询企业推荐信息
     * @param itemId
     * @param vendUU
     * @return
     */
    List<InquiryEnRemind> findByItemIdAndVendUU(Long itemId, Long vendUU);

    /**
     * 通过企业uu查询企业推荐信息
     * @param vendUU
     * @return
     */
    @Query("select itemId from InquiryEnRemind where vendUU = :vendUU")
    List<Long> findIdsByVendUU(@Param("vendUU") Long vendUU);

    /**
     * 根据vendUU获取
     * @param vendUU 报价企业UU
     * @return 企业商机List
     */
    @Query(value = "select o.* from purc$inquiry$enremind o where o.ir_venduu = :vendUU and o.ir_erpdownstatus = 202 limit 200", nativeQuery = true)
    List<InquiryEnRemind> findNotUploadEnRemind(@Param("vendUU") Long vendUU);

    /**
     * 根据itemId查找企业商机
     * @param id  询价明细id
     * @return
     */
    List<InquiryEnRemind> findByItemId(Long id);
}
