package com.uas.ps.inquiry.dao;

import com.uas.ps.inquiry.model.kind.KindConcern;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Set;

/**
 * 类目关注Dao
 * @author dongbw
 */
@Repository
public interface KindConcernDao extends JpaRepository<KindConcern, Long>, JpaSpecificationExecutor<KindConcern> {

    /**
     * 根据企业UU获取本企业所有已收藏类目名称
     * @param enUU  企业UU
     * @return 名称List
     */
    @Query("select k.nameCn from KindConcern k where k.enUU = :enUU")
    List<String> findNameByEnUU(@Param("enUU") Long enUU);

    /**
     * 根据类目名称查询企业列表
     *
     * @param nameCn 类目名称
     * @param enUU 企业uu号
     * @return
     */
    @Query("select k.enUU from KindConcern k where k.nameCn = :nameCn and k.enUU <> :enUU")
    Set<Long> findByNameCnAndNeEnuu(@Param("nameCn") String nameCn, @Param("enUU") Long enUU);
}
