package com.uas.ps.inquiry.dao;

import com.uas.ps.inquiry.model.PublicInquiry;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * 公共询价数据库操作
 *
 * Created by hejq on 2018-01-13.
 */
@Repository
public interface PublicInquiryDao extends JpaSpecificationExecutor<PublicInquiry>, JpaRepository<PublicInquiry, Long> {

    /**
     * 通过企业UU和询价单号查询询价信息
     *
     * @author hejq
     * @date 2018-01-14 12:35
     * @param enuu 企业UU号
     * @param code 询价编号
     * @return
     */
    List<PublicInquiry> findByEnUUAndCode(Long enuu, String code);

    /**
     * 通过来源id和应用来源查询单据
     *
     * @author hejq
     * @date 2018-01-14 14:51
     * @param id 公共询价的主表id
     * @param sourcerapp 应用来源
     * @return
     */
    PublicInquiry findBySourceIdAndSourceApp(Long id, String sourcerapp);

    /**
     * 通过来源id查询报价单主表信息
     *
     * @param id 来源id
     * @return
     */
    PublicInquiry findBySourceId(Long id);
}
