package com.uas.ps.inquiry.model;

import com.uas.ps.core.logging.BufferedLogable;
import org.hibernate.validator.constraints.Length;
import org.springframework.util.StringUtils;

import javax.persistence.*;
import java.io.Serializable;
import java.util.Date;

/**
 * 询价单操作日志
 *
 * Created by hejq on 2018-01-31.
 */
@Table(name = "log$inquiry")
@Entity
public class InquiryLog extends BufferedLogable implements Serializable {

    /**
     * default serialVersionUID
     */
    private static final long serialVersionUID = 1L;

    /**
     * id
     */
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "log_id")
    private Long id;

    /**
     * 操作的企业UU
     */
    @Column(name = "log_enuu")
    private Long enuu;

    /**
     * 用户uu
     */
    @Column(name = "log_useruu")
    private Long useruu;

    /**
     * 应用来源
     */
    @Column(name = "log_source")
    private String source;

    /**
     * 操作时间
     */
    @Column(name = "log_date")
    private Long date;

    /**
     * 日志标题
     */
    @Column(name = "log_title")
    private String title;

    /**
     * 日志消息
     */
    @Column(name = "log_message")
    private String message;

    /**
     * 日志详细描述
     */
    @Column(name = "log_des")
    @Length(max = 1000)
    private String desc;

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public Long getEnuu() {
        return enuu;
    }

    public void setEnuu(Long enuu) {
        this.enuu = enuu;
    }

    public Long getUseruu() {
        return useruu;
    }

    public void setUseruu(Long useruu) {
        this.useruu = useruu;
    }

    public String getSource() {
        return source;
    }

    public void setSource(String source) {
        this.source = source;
    }

    public Long getDate() {
        return date;
    }

    public void setDate(Long date) {
        this.date = date;
    }

    public String getTitle() {
        return title;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public String getMessage() {
        return message;
    }

    public void setMessage(String message) {
        this.message = message;
    }

    public String getDesc() {
        return desc;
    }

    public void setDesc(String desc) {
        this.desc = desc;
    }

    @Override
    public String toString() {
        return "InquiryLog{" +
                "id=" + id +
                ", enuu=" + enuu +
                ", useruu=" + useruu +
                ", source='" + source + '\'' +
                ", date=" + date +
                ", title='" + title + '\'' +
                ", message='" + message + '\'' +
                ", desc='" + desc + '\'' +
                '}';
    }

    /**
     * 日志实体转化为字符串
     *
     * @return
     */
    @Override
    public String bufferedMessage() {
        StringBuffer sb = new StringBuffer();
        sb.append(this.date).append(separator);
        sb.append(this.enuu).append(separator);
        sb.append(this.useruu).append(separator);
        sb.append(this.title).append(separator);
        sb.append(this.message).append(separator);
        sb.append(this.desc == null ? "" : this.desc).append(separator);
        return sb.toString();
    }

    /**
     * 字符串转化为日志实体
     *
     * @param bufferedMessage
     */
    @Override
    public void bufferedLog(String bufferedMessage) {
        String[] strArray = bufferedMessage.split(separator);
        if (strArray.length == 9) {
            this.date = Long.parseLong(strArray[0]);
            this.enuu = Long.parseLong(strArray[2]);
            this.useruu = Long.parseLong(strArray[3]);
            this.title = strArray[4];
            this.message = strArray[5];
            this.desc = StringUtils.hasText(strArray[6]) ? strArray[6] : null;
        }
    }

    public InquiryLog(String title, String message, String detail, Long userUU, Long enUU,  String source) {
        this.title = title;
        this.message = message;
        this.desc = detail;
        this.useruu = userUU;
        this.enuu = enUU;
        this.date = System.currentTimeMillis();
        this.source = source;
    }

    public InquiryLog(String title, String message){
        this.title = title;
        this.message = message;
    }
}
