package com.uas.ps.inquiry.model;


import com.uas.ps.entity.Product;

import javax.persistence.*;
import java.io.Serializable;
import java.util.*;

/**
 * 公共询价单转报价后的询价单明细（查询方法，用明细带出主表信息）
 *
 * @author hejq 2018-01-06
 */
@Table(name = "v$public$inquiryitems")
@Entity
public class PublicInquiryItemInfo implements Serializable {

    /**
     * default serialVersionUID
     */
    private static final long serialVersionUID = 1L;

	/**
	 * id
	 */
	@Id
	@Column(name = "id_id")
	private Long id;

	/**
	 * 来源（买家ERP采购询价明细）的ID
	 */
	@Column(name = "id_sourceid", updatable = false)
	private Long sourceId;

	/**
	 * 序号
	 */
	@Column(name = "id_number")
	private Short number;

	/**
	 * 询价单
	 */
	@ManyToOne(cascade = CascadeType.ALL)
	@JoinColumn(name = "id_inid", nullable = false)
	private PublicInquiryInfo inquiry;

	/**
	 * 买家采购员UU
	 */
	@Column(name = "id_useruu")
	private Long userUU;

	/**
	 * 联系人姓名
	 */
	@Column(name = "id_username")
	private String userName;

	/**
	 * 联系人电话
	 */
	@Column(name = "id_usertel")
	private String userTel;

    /**
     * 产品id
     */
	@Column(name = "id_prid")
	private Long productId;

	/**
	 * 产品
	 */
	@Transient
	private Product product;

	/**
	 * 币种
	 */
	@Column(name = "id_currency")
	private String currency;

	/**
	 * 税率
	 */
	@Column(name = "id_taxrate")
	private Float taxrate;

	/**
	 * 供应商UU
	 */
	@Column(name = "id_venduu")
	private Long vendUU;

	/**
	 * 供应商企业信息
	 */
	@OneToOne(cascade = { CascadeType.MERGE, CascadeType.REFRESH })
	@JoinColumn(name = "id_venduu", insertable = false, updatable = false)
	private Enterprise enterprise;

	/**
	 * 供应商联系人UU
	 */
	@Column(name = "id_venduseruu")
	private Long vendUserUU;

	/**
	 * （买家预先提供的）有效期始
	 */
	@Column(name = "id_fromdate")
	private Date fromDate;

	/**
	 * （卖家报的）最小订购量
	 */
	@Column(name = "id_minorderqty")
	private Double minOrderQty;

	/**
	 * （卖家报的）最小包装量
	 */
	@Column(name = "id_minpackqty")
	private Double minPackQty;

	/**
	 * （卖家报的）供应商物料编号
	 */
	@Column(name = "id_vendorprodcode")
	private String vendorprodcode;

	/**
	 * （卖家报的）交货周期（天数）
	 */
	@Column(name = "id_leadtime")
	private Long leadtime;

	/**
	 * {未回复、已回复}
	 */
	@Column(name = "id_status")
	private Short status;

	/**
	 * (针对卖家的)询价传输状态{待上传、已下载}
	 */
	@Column(name = "id_sendstatus")
	private Short sendStatus;

	/**
	 * (针对买家的)报价信息传输状态{待上传、已下载}
	 */
	@Column(name = "id_backstatus")
	private Short backStatus;

	/**
	 * (针对卖家的)报价信息传输状态{待上传、已下载}
	 */
	@Column(name = "id_replysendstatus")
	private Short replySendStatus;

	/**
	 * 是否采纳
	 */
	@Column(name = "id_agreed")
	private Short agreed;

	/**
	 * (针对卖家的)是否采纳信息传输状态{待上传、已下载}
	 */
	@Column(name = "id_decidestatus")
	private Short decideStatus;

	/**
	 * (针对卖家的)作废信息传输状态{待上传、已下载}
	 */
	@Column(name = "id_invalidstatus")
	private Short invalidStatus;

	/**
	 * 录入时间
	 *
	 */
	@Column(name = "id_date")
	private Date date;

	/**
	 *  报价是否过期
	 */
	@Column(name = "id_overdue", insertable = false, updatable = false)
	private Short overdue;

	/**
	 * 应用来源ERP、B2B
	 */
	@Column(name = "id_sourceapp")
	private String source;

	/**
	 * 报价的应用
	 */
	@Column(name = "id_qutoapp")
	private String qutoApp;

	/**
	 * 需求数量
	 */
	@Column(name = "id_needquantity")
	private Double needquantity;

    /**
     * erp传输状态
     *
     * <pre>erp发出数据传输请求时，赋给状态，完成后更新状态</pre>
     *
     * 1、 传输完成<br>
     * 0、 正在传输
     *
     */
	@Column(name = "id_erpstatus")
    private Short erpstatus;

	/*这下面是供应商报价时存的相关信息；
	* 因为存在非客户报价，而且公共服务里面没有企业信息，现存入相关字段，后续处理*/
	/**
	 * 供应商名称
	 */
	@Column(name = "id_vend_name")
	private String vendName;

	/**
	 * 供应商营业执照
	 */
	@Column(name = "id_vend_businesscode")
	private String businessCode;

	/**
	 * 报价时间
	 */
	@Column(name = "id_offertime")
	private Date offerTime;

    /**
     * 分段报价明细
     */
    @OneToMany(cascade = CascadeType.ALL, fetch = FetchType.EAGER)
    @JoinColumn(name = "ir_idid")
    @OrderBy("lapQty")
    private List<PublicInquiryReplyInfo> replies;

    /**
     * 截止日期
     */
    @Column(name = "id_enddate")
    private Date endDate;


    /**
     * ************* 上传的物料信息，物料冗余字段 *************
     */
    /**
     * 物料编号
     */
    @Column(name = "id_prodcode")
    private String prodCode;

    /**
     * 名称
     */
    @Column(name = "id_prodtitle")
    private String prodTitle;

    /**
     * 规格
     */
    @Column(name = "id_prodspec")
    private String spec;

    /**
     * 型号
     */
    @Column(name = "id_cmpcode")
    private String cmpCode;

    /**
     * 品牌
     */
    @Column(name = "id_inbrand")
    private String inbrand;
    /**
     * ******end*******
     */

    /**
     * 剩余时间
     */
    @Transient
    private Long remainingTime;

	/**
	 * 报价附件信息
	 */
	@Transient
	private String attachUrl;

    /**
     * 报价附件名称
     */
    @Transient
    private String attachName;

	/**
	 * 供应商企业信息
	 */
	@OneToOne(cascade = { CascadeType.MERGE, CascadeType.REFRESH })
	@JoinColumn(name = "id_venduseruu", insertable = false, updatable = false)
	private User user;

	/**
	 * 买家发出公共询价单时录入的币别
	 */
	@Column(name = "id_cust_currency")
	private String custCurrency;

	/**
	 * 拒绝采纳理由
	 */
	@Column(name = "id_refusereason")
	private String refusereason;

	/******** 替代物料信息  */

	/**
	 * 替代型号
	 */
	@Column(name = "id_replace_cmpcode")
	private String replaceCmpCode;

	/**
	 * 替代规格
	 */
	@Column(name = "id_replace_spec")
	private String replaceSpec;

	/**
	 * 替代品牌
	 */
	@Column(name = "id_replace_brand")
	private String replaceBrand;

	/**
	 * 是否替代物料报价  1 是替代料报价， 0为普通报价
	 */
	@Column(name = "id_isreplace")
	private Short isReplace = 0;

	/**
	 * 总计价格（用于商城bom求购明细价格分析  dongbw  2018年9月5日 14:59:48）
	 */
	@Column(name = "id_sumprice", columnDefinition = "double default 0")
	private Double sumPrice;

	/** end  */

	public User getUser() {
		return user;
	}

	public void setUser(User user) {
		this.user = user;
	}

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public Long getSourceId() {
		return sourceId;
	}

	public void setSourceId(Long sourceId) {
		this.sourceId = sourceId;
	}

	public Short getNumber() {
		return number;
	}

	public void setNumber(Short number) {
		this.number = number;
	}

	public PublicInquiryInfo getInquiry() {
		return inquiry;
	}

	public void setInquiry(PublicInquiryInfo inquiry) {
		this.inquiry = inquiry;
	}

	public Long getProductId() {
		return productId;
	}

	public void setProductId(Long productId) {
		this.productId = productId;
	}

	public Product getProduct() {
		Product product = new Product();
		product.setId(this.productId);
		product.setCode(this.prodCode);
		product.setSpec(this.spec);
		product.setCmpCode(this.cmpCode);
		product.setpCmpCode(this.cmpCode);
		product.setTitle(this.prodTitle);
		product.setBrand(this.inbrand);
		product.setpBrandEn(this.inbrand);
		return product;
	}

	public void setProduct(Product product) {
		this.product = product;
	}

	public String getProdCode() {
		return prodCode;
	}

	public void setProdCode(String prodCode) {
		this.prodCode = prodCode;
	}

	public String getCurrency() {
		return currency;
	}

	public void setCurrency(String currency) {
		this.currency = currency;
	}

	public Float getTaxrate() {
		return taxrate;
	}

	public void setTaxrate(Float taxrate) {
		this.taxrate = taxrate;
	}

	public Date getFromDate() {
		return fromDate;
	}

	public void setFromDate(Date fromDate) {
		this.fromDate = fromDate;
	}

	public Long getVendUU() {
		return vendUU;
	}

	public void setVendUU(Long vendUU) {
		this.vendUU = vendUU;
	}

	public Enterprise getEnterprise() {
		return enterprise;
	}

	public void setEnterprise(Enterprise enterprise) {
		this.enterprise = enterprise;
	}

	public Long getVendUserUU() {
		return vendUserUU;
	}

	public void setVendUserUU(Long vendUserUU) {
		this.vendUserUU = vendUserUU;
	}

	public Short getAgreed() {
		return agreed;
	}

	public void setAgreed(Short agreed) {
		this.agreed = agreed;
	}

	public Short getStatus() {
		return status;
	}

	public void setStatus(Short status) {
		this.status = status;
	}

	public Short getSendStatus() {
		return sendStatus;
	}

	public void setSendStatus(Short sendStatus) {
		this.sendStatus = sendStatus;
	}

	public Double getMinOrderQty() {
		return minOrderQty;
	}

	public void setMinOrderQty(Double minOrderQty) {
		this.minOrderQty = minOrderQty;
	}

	public Double getMinPackQty() {
		return minPackQty;
	}

	public void setMinPackQty(Double minPackQty) {
		this.minPackQty = minPackQty;
	}

	public Short getBackStatus() {
		return backStatus;
	}

	public void setBackStatus(Short backStatus) {
		this.backStatus = backStatus;
	}

	public Short getReplySendStatus() {
		return replySendStatus;
	}

	public void setReplySendStatus(Short replySendStatus) {
		this.replySendStatus = replySendStatus;
	}

	public Short getDecideStatus() {
		return decideStatus;
	}

	public void setDecideStatus(Short decideStatus) {
		this.decideStatus = decideStatus;
	}

	public Long getUserUU() {
		return userUU;
	}

	public void setUserUU(Long userUU) {
		this.userUU = userUU;
	}

	public String getUserName() {
		return userName;
	}

	public void setUserName(String userName) {
		this.userName = userName;
	}

	public String getUserTel() {
		return userTel;
	}

	public void setUserTel(String userTel) {
		this.userTel = userTel;
	}

	public String getVendorprodcode() {
		return vendorprodcode;
	}

	public void setVendorprodcode(String vendorprodcode) {
		this.vendorprodcode = vendorprodcode;
	}

	public Long getLeadtime() {
		return leadtime;
	}

	public void setLeadtime(Long leadtime) {
		this.leadtime = leadtime;
	}

	public Date getDate() {
		return date;
	}

	public void setDate(Date date) {
		this.date = date;
	}

	public String getSource() {
		return source;
	}

	public void setSource(String source) {
		this.source = source;
	}

	public String getQutoApp() {
		return qutoApp;
	}

	public void setQutoApp(String qutoApp) {
		this.qutoApp = qutoApp;
	}

	public Short getInvalidStatus() {
		return invalidStatus;
	}

	public void setInvalidStatus(Short invalidStatus) {
		this.invalidStatus = invalidStatus;
	}

	public Double getNeedquantity() {
		return needquantity;
	}

	public void setNeedquantity(Double needquantity) {
		this.needquantity = needquantity;
	}

	public Short getOverdue() {
		return overdue;
	}

	public void setOverdue(Short overdue) {
		this.overdue = overdue;
	}

    public Short getErpstatus() {
        return erpstatus;
    }

    public void setErpstatus(Short erpstatus) {

	    this.erpstatus = erpstatus ;
    }

	public String getVendName() {
		return vendName;
	}

	public void setVendName(String vendName) {
		this.vendName = vendName;
	}

	public String getBusinessCode() {
		return businessCode;
	}

	public void setBusinessCode(String businessCode) {
		this.businessCode = businessCode;
	}

	public Date getOfferTime() {
		return offerTime;
	}

	public void setOfferTime(Date offerTime) {
		this.offerTime = offerTime;
	}

    public List<PublicInquiryReplyInfo> getReplies() {
        return replies;
    }

    public void setReplies(List<PublicInquiryReplyInfo> replies) {
        this.replies = replies;
    }

    public Date getEndDate() {
        return endDate;
    }

    public void setEndDate(Date endDate) {
        this.endDate = endDate;
    }

    public String getProdTitle() {
        return prodTitle;
    }

    public void setProdTitle(String prodTitle) {
        this.prodTitle = prodTitle;
    }

    public String getSpec() {
        return spec;
    }

    public void setSpec(String spec) {
        this.spec = spec;
    }

    public String getCmpCode() {
        return cmpCode;
    }

    public void setCmpCode(String cmpCode) {
        this.cmpCode = cmpCode;
    }

    public String getInbrand() {
        return inbrand;
    }

    public void setInbrand(String inbrand) {
        this.inbrand = inbrand;
    }

    public Long getRemainingTime() {
        if (null != endDate) {
            return endDate.getTime() - System.currentTimeMillis();
        }
        return null;
    }

    public String getAttachUrl() {
        return attachUrl;
    }

    public void setAttachUrl(String attachUrl) {
        this.attachUrl = attachUrl;
    }

    public String getAttachName() {
        return attachName;
    }

    public void setAttachName(String attachName) {
        this.attachName = attachName;
    }

	public String getCustCurrency() {
		return custCurrency;
	}

	public void setCustCurrency(String custCurrency) {
		this.custCurrency = custCurrency;
	}

	public void setRemainingTime(Long remainingTime) {
		this.remainingTime = remainingTime;
	}

	public String getRefusereason() {
		return refusereason;
	}

	public void setRefusereason(String refusereason) {
		this.refusereason = refusereason;
	}

	public String getReplaceCmpCode() {
		return replaceCmpCode;
	}

	public void setReplaceCmpCode(String replaceCmpCode) {
		this.replaceCmpCode = replaceCmpCode;
	}

	public String getReplaceSpec() {
		return replaceSpec;
	}

	public void setReplaceSpec(String replaceSpec) {
		this.replaceSpec = replaceSpec;
	}

	public String getReplaceBrand() {
		return replaceBrand;
	}

	public void setReplaceBrand(String replaceBrand) {
		this.replaceBrand = replaceBrand;
	}

	public Short getIsReplace() {
		return isReplace;
	}

	public void setIsReplace(Short isReplace) {
		this.isReplace = isReplace;
	}

	public Double getSumPrice() {
		return sumPrice;
	}

	public void setSumPrice(Double sumPrice) {
		this.sumPrice = sumPrice;
	}

	@Override
	public String toString() {
		return "PublicInquiryItemInfo{" +
				"id=" + id +
				", sourceId=" + sourceId +
				", number=" + number +
				", inquiry=" + inquiry +
				", userUU=" + userUU +
				", userName='" + userName + '\'' +
				", userTel='" + userTel + '\'' +
				", productId=" + productId +
				", product=" + product +
				", currency='" + currency + '\'' +
				", taxrate=" + taxrate +
				", vendUU=" + vendUU +
				", enterprise=" + enterprise +
				", vendUserUU=" + vendUserUU +
				", fromDate=" + fromDate +
				", minOrderQty=" + minOrderQty +
				", minPackQty=" + minPackQty +
				", vendorprodcode='" + vendorprodcode + '\'' +
				", leadtime=" + leadtime +
				", status=" + status +
				", sendStatus=" + sendStatus +
				", backStatus=" + backStatus +
				", replySendStatus=" + replySendStatus +
				", agreed=" + agreed +
				", decideStatus=" + decideStatus +
				", invalidStatus=" + invalidStatus +
				", date=" + date +
				", overdue=" + overdue +
				", source='" + source + '\'' +
				", qutoApp='" + qutoApp + '\'' +
				", needquantity=" + needquantity +
				", erpstatus=" + erpstatus +
				", vendName='" + vendName + '\'' +
				", businessCode='" + businessCode + '\'' +
				", offerTime=" + offerTime +
				", replies=" + replies +
				", endDate=" + endDate +
				", prodCode='" + prodCode + '\'' +
				", prodTitle='" + prodTitle + '\'' +
				", spec='" + spec + '\'' +
				", cmpCode='" + cmpCode + '\'' +
				", inbrand='" + inbrand + '\'' +
				", remainingTime=" + remainingTime +
				", attachUrl='" + attachUrl + '\'' +
				", attachName='" + attachName + '\'' +
				", user=" + user +
				", custCurrency='" + custCurrency + '\'' +
				", refusereason='" + refusereason + '\'' +
				", replaceCmpCode='" + replaceCmpCode + '\'' +
				", replaceSpec='" + replaceSpec + '\'' +
				", replaceBrand='" + replaceBrand + '\'' +
				", isReplace=" + isReplace +
				'}';
	}
}
