package com.uas.ps.inquiry.model;

import com.alibaba.fastjson.annotation.JSONField;
import com.fasterxml.jackson.annotation.JsonIgnore;

import javax.persistence.*;
import java.io.Serializable;
import java.util.Date;
import java.util.Set;

/**
 * 公共库新增的询价信息表中间表（查询信息）
 *
 * @author hejq
 * @date 2018-01-17 15:46
 */
@Entity
@Table(name = "purc$puinquiry")
public class PurcInquiryInfo implements Serializable {

    /**
     * default serialVersionUID
     */
    private static final long serialVersionUID = 1L;

    /**
     * id
     */
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "in_id")
    private Long id;

    /**
     * 询价单所属企业UU
     */
    @Column(name = "in_enuu")
    private Long enUU;

    /**
     * 企业名称（方便搜索过滤）
     */
    @Column(name = "in_enname")
    private String enName;

    /**
     * 询价企业
     */
    @OneToOne(cascade = { CascadeType.MERGE, CascadeType.REFRESH })
    @JoinColumn(name = "in_enuu", insertable = false, updatable = false)
    private Enterprise enterprise;

    /**
     * 询价单所属用户UU
     */
    @Column(name = "in_recorderuu")
    private Long recorderUU;

    /**
     * 流水号
     */
    @Column(name = "in_code")
    private String code;

    /**
     * 单据归属日期
     */
    @Column(name = "in_date")
    private Date date;

    /**
     * 录入人
     */
    @Column(name = "in_recorder")
    private String recorder;

    /**
     * 报价截止日期
     */
    @Column(name = "in_enddate")
    private Date endDate;

    /**
     * 备注
     */
    @Column(name = "in_remark")
    private String remark;

    /**
     * 环保要求
     */
    @Column(name = "in_environment")
    private String environment;

    /**
     * 询价明细
     */
    @OneToMany(mappedBy = "inquiry", cascade = { CascadeType.REFRESH }, fetch = FetchType.EAGER)
    @OrderBy("number")
    private Set<PurcInquiryItemInfo> inquiryItems;

    /**
     * 附件
     */
    @OneToMany(fetch = FetchType.EAGER, cascade = { CascadeType.ALL })
    @JoinTable(name = "purc$puinquiryattach", joinColumns = @JoinColumn(name = "in_id", referencedColumnName = "in_id") , inverseJoinColumns = @JoinColumn(name = "at_id", referencedColumnName = "at_id") )
    private Set<Attach> attachs;

    /**
     * 单据状态（已提交、在录入)
     */
    @Column(name = "in_enterystatus")
    private Integer enteryStatus;

    /**
     * 收货地址
     */
    @Column(name = "in_ship")
    private String ship;

    /**
     * 是否开票<br>
     * 1. 是<br>
     * 0. 否
     */
    @Column(name = "in_invoice")
    private Short invoice;

    /**
     * 是否含税
     */
    @Column(name = "in_iftax")
    private Short ifTax;

    /**
     * 币别
     */
    @Column(name = "in_currency")
    private String currency;

    /**
     * 询价类型
     */
    @Column(name = "in_inquirytype")
    private String inquirytype;

    /**
     * 应用来源
     */
    @Column(name = "in_sourceapp")
    private String sourceapp;

    /**
     * 如果从erp传过来的，记录id
     */
    @Column(name = "in_erpid")
    private Long erpid;

    /**
     * erp传过来的记录时间
     */
    @Column(name = "in_erpdate")
    private  Date erpdate;

    /**
     * 明细总数
     */
    @Column(name = "in_amount")
    private Integer amount;

    /**
     * bom单规格（仅bom单求购有）
     */
    @Column(name = "in_spec", columnDefinition = "TEXT", length = 500)
    private String spec;

    /**
     * 采购套数（bom询价使用，普通询价默认1）
     */
    @Column(name = "in_count")
    private Integer count = 1;

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public Long getEnUU() {
        return enUU;
    }

    public void setEnUU(Long enUU) {
        this.enUU = enUU;
    }

    public String getEnName() {
        return enName;
    }

    public void setEnName(String enName) {
        this.enName = enName;
    }

    public Enterprise getEnterprise() {
        return enterprise;
    }

    public void setEnterprise(Enterprise enterprise) {
        this.enterprise = enterprise;
    }

    public Long getRecorderUU() {
        return recorderUU;
    }

    public void setRecorderUU(Long recorderUU) {
        this.recorderUU = recorderUU;
    }

    public String getCode() {
        return code;
    }

    public void setCode(String code) {
        this.code = code;
    }

    public Date getDate() {
        return date;
    }

    public void setDate(Date date) {
        this.date = date;
    }

    public String getRecorder() {
        return recorder;
    }

    public void setRecorder(String recorder) {
        this.recorder = recorder;
    }

    public Date getEndDate() {
        return endDate;
    }

    public void setEndDate(Date endDate) {
        this.endDate = endDate;
    }

    public String getRemark() {
        return remark;
    }

    public void setRemark(String remark) {
        this.remark = remark;
    }

    public String getEnvironment() {
        return environment;
    }

    public void setEnvironment(String environment) {
        this.environment = environment;
    }

    @JsonIgnore
    @JSONField(serialize = false)
    public Set<PurcInquiryItemInfo> getInquiryItems() {
        return inquiryItems;
    }

    public void setInquiryItems(Set<PurcInquiryItemInfo> inquiryItems) {
        this.inquiryItems = inquiryItems;
    }

    public Set<Attach> getAttachs() {
        return attachs;
    }

    public void setAttachs(Set<Attach> attachs) {
        this.attachs = attachs;
    }

    public Integer getEnteryStatus() {
        return enteryStatus;
    }

    public void setEnteryStatus(Integer enteryStatus) {
        this.enteryStatus = enteryStatus;
    }

    public String getShip() {
        return ship;
    }

    public void setShip(String ship) {
        this.ship = ship;
    }

    public Short getInvoice() {
        return invoice;
    }

    public void setInvoice(Short invoice) {
        this.invoice = invoice;
    }

    public Short getIfTax() {
        return ifTax;
    }

    public void setIfTax(Short ifTax) {
        this.ifTax = ifTax;
    }

    public String getCurrency() {
        return currency;
    }

    public void setCurrency(String currency) {
        this.currency = currency;
    }

    public String getInquirytype() {
        return inquirytype;
    }

    public void setInquirytype(String inquirytype) {
        this.inquirytype = inquirytype;
    }

    public String getSourceapp() {
        return sourceapp;
    }

    public void setSourceapp(String sourceapp) {
        this.sourceapp = sourceapp;
    }

    public Long getErpid() {
        return erpid;
    }

    public void setErpid(Long erpid) {
        this.erpid = erpid;
    }

    public Date getErpdate() {
        return erpdate;
    }

    public void setErpdate(Date erpdate) {
        this.erpdate = erpdate;
    }

    public Integer getAmount() {
        return amount;
    }

    public void setAmount(Integer amount) {
        this.amount = amount;
    }

    public String getSpec() {
        return spec;
    }

    public void setSpec(String spec) {
        this.spec = spec;
    }

    public Integer getCount() {
        return count;
    }

    public void setCount(Integer count) {
        this.count = count;
    }
}
