package com.uas.ps.inquiry.model;

import com.alibaba.fastjson.annotation.JSONField;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.uas.ps.entity.Product;

import javax.persistence.*;
import java.util.*;

/**
 * 平台新增询价单明细
 * 
 * @author hejq
 * @date 2018-01-13 19:19
 */
@Entity
@Table(name = "purc$puinquiryitems")
public class PurcInquiryItem {

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	@Column(name = "id_id")
	private Long id;

	/**
	 * 序号
	 */
	@Column(name = "id_number")
	private Short number;

	/**
	 * 询价单
	 */
	@ManyToOne(cascade = CascadeType.ALL, optional = true)
	@JoinColumn(name = "id_inid", nullable = false)
	private PurcInquiry inquiry;

	/**
	 * 买家采购员UU
	 */
	@Column(name = "id_useruu")
	private Long userUU;

	/**
	 * 联系人姓名
	 */
	@Column(name = "id_username")
	private String userName;

	/**
	 * 联系人电话
	 */
	@Column(name = "id_usertel")
	private String userTel;

	/**
	 * 产品id
	 */
	@Column(name = "id_prid")
	private Long productId;

	/**
	 * 产品
	 */
//	@OneToOne(cascade = {CascadeType.ALL})
//	@JoinColumn(name = "id_prid", insertable = false, updatable = false)
	@Transient
	private Product product;

	/**
	 * ************* 上传的物料信息，因为可能是手动输入的，物料款不存在，需要先进行保存物料 *************
	 */
	/**
	 * 物料编号
	 */
	@Column(name = "id_prodcode")
	private String prodCode;

	/**
	 * 名称
	 */
	@Column(name = "id_prodtitle")
	private String prodTitle;

	/**
	 * 规格
	 */
	@Column(name = "id_prodspec")
	private String spec;

	/**
	 * 型号
	 */
	@Column(name = "id_cmpcode")
	private String cmpCode;

	/**
	 * 品牌
	 */
	@Column(name = "id_inbrand")
	private String inbrand;
	/**
	 * ******end*******
	 */

	/**
	 * 币种
	 */
	@Column(name = "id_currency")
	private String currency;

	/**
	 * 税率
	 */
	@Column(name = "id_taxrate")
	private Float taxrate;

	/**
	 * （买家预先提供的）有效期始
	 */
	@Column(name = "id_fromdate")
	private Date fromDate;

	/**
	 * 分段报价明细
	 */
	@OneToMany(cascade = CascadeType.ALL, fetch = FetchType.EAGER)
	@JoinColumn(name = "ir_idid")
	@OrderBy("lapQty")
	private List<PurchaseInquiryReply> replies;

	/**
	 * {未回复、已回复}
	 */
	@Column(name = "id_status")
	private Short status;

	/**
	 * 是否采纳
	 */
	@Column(name = "id_agreed")
	private Short agreed;

	/**
	 * 是否已过期 1为已过期，0为未过期 已过期单据无法报价
	 * 
	 * @return
	 */
	@Column(name = "id_overdue")
	private Short overdue;

	/**
	 * 来源ERP或B2B
	 */
	@Column(name = "id_source")
	private String source;

	/**
	 * 需求数量
	 */
	@Column(name = "id_needquantity")
	private Double needquantity;

	/**
	 * 来源id，可能是ERP同步过来的单据
	 */
	@Column(name = "id_sourceid")
	private Long sourceid;

    /**
     * 单据日期，索引过滤需要
     */
	@Column(name = "id_date")
    private Date date;

	/**
	 * 发布询价企业uu
	 */
	@Column(name = "id_enuu")
	private Long enuu;

    /**
     * 物料报价数量（记录多少人或企业对该物料进行报价）
     */
	@Column(name = "id_offeramount")
    private Integer offerAmount;

    /**
     * 截止日期
     */
	@Column(name = "id_enddate")
    private Date endDate;

	/**
	 * 剩余时间
	 */
	@Transient
	private Long remainingTime;

	/**
	 * 是否有效  1有效  0 无效
	 */
	@Column(name = "id_valid", columnDefinition = "TINYINT DEFAULT 1")
	private Integer valid;

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public Short getNumber() {
		return number;
	}

	public void setNumber(Short number) {
		this.number = number;
	}

	@JsonIgnore
	@JSONField(serialize = false)
	public PurcInquiry getInquiry() {
		return inquiry;
	}

	public void setInquiry(PurcInquiry inquiry) {
		this.inquiry = inquiry;
	}

	public Long getUserUU() {
		return userUU;
	}

	public void setUserUU(Long userUU) {
		this.userUU = userUU;
	}

	public String getUserName() {
		return userName;
	}

	public void setUserName(String userName) {
		this.userName = userName;
	}

	public String getUserTel() {
		return userTel;
	}

	public void setUserTel(String userTel) {
		this.userTel = userTel;
	}

    public Long getProductId() {
		return productId;
	}

	public void setProductId(Long productId) {
		this.productId = productId;
	}

    public Product getProduct() {
        return product;
    }

    public void setProduct(Product product) {
        this.product = product;
    }

    public String getCurrency() {
		return currency;
	}

	public void setCurrency(String currency) {
		this.currency = currency;
	}

	public Float getTaxrate() {
		return taxrate;
	}

	public void setTaxrate(Float taxrate) {
		this.taxrate = taxrate;
	}

	public Date getFromDate() {
		return fromDate;
	}

	public void setFromDate(Date fromDate) {
		this.fromDate = fromDate;
	}

	public List<PurchaseInquiryReply> getReplies() {
		return replies;
	}

	public void setReplies(List<PurchaseInquiryReply> replies) {
		this.replies = replies;
	}

	public Short getStatus() {
		return status;
	}

	public void setStatus(Short status) {
		this.status = status;
	}

	public Short getAgreed() {
		return agreed;
	}

	public void setAgreed(Short agreed) {
		this.agreed = agreed;
	}

	public Short getOverdue() {
		return overdue;
	}

	public void setOverdue(Short overdue) {
		this.overdue = overdue;
	}

	public String getSource() {
		return source;
	}

	public void setSource(String source) {
		this.source = source;
	}

	public String getProdCode() {
		return prodCode;
	}

	public void setProdCode(String prodCode) {
		this.prodCode = prodCode;
	}

	public String getProdTitle() {
		return prodTitle;
	}

	public void setProdTitle(String prodTitle) {
		this.prodTitle = prodTitle;
	}

	public String getSpec() {
		return spec;
	}

	public void setSpec(String spec) {
		this.spec = spec;
	}

	public String getCmpCode() {
		return cmpCode;
	}

	public void setCmpCode(String cmpCode) {
		this.cmpCode = cmpCode;
	}

	public String getInbrand() {
		return inbrand;
	}

	public void setInbrand(String inbrand) {
		this.inbrand = inbrand;
	}

	public Double getNeedquantity() {
		return needquantity;
	}

	public void setNeedquantity(Double needquantity) {
		this.needquantity = needquantity;
	}

	public Long getSourceid() {
		return sourceid;
	}

	public void setSourceid(Long sourceid) {
		this.sourceid = sourceid;
	}

    public Date getDate() {
        return date;
    }

    public void setDate(Date date) {
        this.date = date;
    }

	public Long getEnuu() {
		return enuu;
	}

	public void setEnuu(Long enuu) {
		this.enuu = enuu;
	}

    public Integer getOfferAmount() {
        return offerAmount;
    }

    public void setOfferAmount(Integer offerAmount) {
        this.offerAmount = offerAmount;
    }

    public Date getEndDate() {
        return endDate;
    }

    public void setEndDate(Date endDate) {
        this.endDate = endDate;
    }

	public Long getRemainingTime() {
		if (endDate != null) {
			return endDate.getTime() - System.currentTimeMillis();
		}
		return null;
	}

	public void setRemainingTime(Long remainingTime) {
		this.remainingTime = remainingTime;
	}

	@Override
	public String toString() {
		return "PurcInquiryItem{" +
				"id=" + id +
				", number=" + number +
				", inquiry=" + inquiry +
				", userUU=" + userUU +
				", userName='" + userName + '\'' +
				", userTel='" + userTel + '\'' +
				", productId=" + productId +
				", product=" + product +
				", prodCode='" + prodCode + '\'' +
				", prodTitle='" + prodTitle + '\'' +
				", spec='" + spec + '\'' +
				", cmpCode='" + cmpCode + '\'' +
				", inbrand='" + inbrand + '\'' +
				", currency='" + currency + '\'' +
				", taxrate=" + taxrate +
				", fromDate=" + fromDate +
				", replies=" + replies +
				", status=" + status +
				", agreed=" + agreed +
				", overdue=" + overdue +
				", source='" + source + '\'' +
				", needquantity=" + needquantity +
				", sourceid=" + sourceid +
				", date=" + date +
				", enuu=" + enuu +
				", offerAmount=" + offerAmount +
				", endDate=" + endDate +
				", remainingTime=" + remainingTime +
				", valid=" + valid +
				'}';
	}

	public Integer getValid() {
		return valid;
	}

	public void setValid(Integer valid) {
		this.valid = valid;
	}

}
