package com.uas.ps.inquiry.service;

import com.uas.ps.inquiry.entity.InquiryDetailInfo;
import com.uas.ps.inquiry.entity.InquiryProductInfo;
import com.uas.ps.inquiry.model.*;
import com.uas.ps.inquiry.page.PageInfo;
import com.uas.ps.inquiry.page.SearchFilter;
import javassist.NotFoundException;
import org.springframework.data.domain.Page;

import java.util.Date;
import java.util.List;

/**
 * 转询价报价后的数据查询
 *
 * Created by hejq on 2018-01-17.
 */
public interface InquiryService {

    /**
     * 查询公共询价列表信息
     * @param info 分页新
     * @param filter 过滤条件
     * @param state 过滤状态
     * @param overdue 是否过期
     * @return
     */
    Page<PurcInquiryItemInfo> findTodoByPageInfo(PageInfo info, SearchFilter filter, String state, Integer overdue);

    /**
     * 保存公共询价
     *
     * @param inquiry 询价信息
     */
    PurcInquiry saveInquiry(PurcInquiry inquiry) throws NotFoundException;

    /**
     * 通过报价明细id对供应商报价进行相关审核操作
     *
     * @param id 报价明细id
     * @param status 状态
     */
    void decideQuote(Long id, Short status);

    /**
     * 通过报价明细id对供应商报价进行相关审核操作
     *
     * @param id 报价明细id
     * @param status 状态
     */
    void adpotQuote(Long id, Short status);

    /**
     * 通过报价明细id对供应商报价进行相关审核操作
     *
     * @param id 报价明细id
     * @param status 状态
     * @param refusereason 拒绝理由
     */
    void refuseQuote(Long id, Short status,String refusereason);

    /**
     * 针对客户，查询供应商报价详情
     *
     * @param id 主表id
     * @param enuu 企业UU
     * @return
     */
    InquiryDetailInfo findById(Long id, Long enuu);

    /**
     * 通过企业UU和分页信息等查询已发布信息
     *
     * @param pageInfo 分页信息
     * @param searchFilter 过滤条件
     * @return
     */
    Page<PurcInquiry> findByPageInfo(PageInfo pageInfo, SearchFilter searchFilter);

    /**
     * 针对客户，查询单个物料的报价情况
     *
     * @param id 明细id
     * @param enuu 当前企业UU
     * @return
     */
    InquiryProductInfo findInquiryDetailById(Long id, Long enuu);

    /**
     * 针对客户查询供应商报价信息
     *
     * @param pageInfo 分页信息
     * @param searchFilter 过滤条件
     * @return
     */
    Page<PurcInquiryItemInfo> findQuotationsByPage(PageInfo pageInfo, SearchFilter searchFilter, Long enUU, Long userUU, Short overdue);

    /**
     * 针对客户单个公共询价，查询供应商报价信息
     * @param id
     * @return
     */
    PurcInquiryItemInfo findQuotationById(Long id);

    /**
     * 测试消息
     * @param startTime
     * @param endTime
     * @param useruu
     * @param enuu
     * @return
     */
    List<InquiryRemind> testMessage(Date startTime, Date endTime, Long useruu, Long enuu);

    /**
     * 根据商机发送消息
     * @param reminds 商机List
     */
    void sendMessage(List<InquiryRemind> reminds);

    /**
     * 以供应商企业UU和用户UU分组查询商机
     * @param hours  查之前多少小时内的单据
     * @return 商机List
     */
    List<InquiryRemind> findInquiryRemindGroupByVendUUAndVendUserUU(int hours);

    /**
     * 讲报价产品数设置进page
     * @param page  询价单page
     * @return 询价单pege
     */
    Page<PurcInquiry> setQuotedAmount(Page<PurcInquiry> page);
}
