package com.uas.ps.inquiry.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.uas.ps.core.util.ContextUtils;
import com.uas.ps.entity.Product;
import com.uas.ps.entity.Status;
import com.uas.ps.inquiry.AccessConfiguration;
import com.uas.ps.inquiry.dao.*;
import com.uas.ps.inquiry.domain.IPage;
import com.uas.ps.inquiry.entity.Constant;
import com.uas.ps.inquiry.entity.InquirySource;
import com.uas.ps.inquiry.model.*;
import com.uas.ps.inquiry.page.PageInfo;
import com.uas.ps.inquiry.page.SearchFilter;
import com.uas.ps.inquiry.page.criteria.CriterionExpression;
import com.uas.ps.inquiry.page.criteria.LogicalExpression;
import com.uas.ps.inquiry.page.criteria.PredicateUtils;
import com.uas.ps.inquiry.page.criteria.SimpleExpression;
import com.uas.ps.inquiry.page.exception.IllegalOperatorException;
import com.uas.ps.inquiry.service.InquiryForSaleService;
import com.uas.ps.inquiry.util.FlexJsonUtils;
import com.uas.ps.inquiry.util.HttpUtil;
import com.uas.ps.inquiry.util.IPageUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.ui.ModelMap;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 针对卖家，对询价的相关操作
 *
 * Created by hejq on 2018-01-18.
 */
@Service
@Transactional
public class InquiryForSaleServiceImpl implements InquiryForSaleService {

    @Autowired
    private PublicInquiryItemDao infoDao;

    @Autowired
    private AttachDao attachDao;

    @Autowired
    private JdbcTemplate jdbcTemplate;

    @Autowired
    private PublicInquiryDao inquiryDao;

    @Autowired
    private PurcInquiryItemInfoDao inquiryItemInfoDao;

    @Autowired
    private EnterpriseDao enterpriseDao;

    @Autowired
    private PurcInquiryItemDao purcInquiryItemDao;

    @Autowired
    private InquiryRemindDao remindDao;

    @Autowired
    private ProductDao productDao;

    @Autowired
    private InquiryEnRemindDao inquiryEnRemindDao;

    /**
     * 日志
     */

    private final static Logger log = LoggerFactory.getLogger(InquiryForSaleServiceImpl.class);

    /**
     * 公共物料访问地址
     */
    private final String PS_PRODUCT_URL = ContextUtils.getBean(AccessConfiguration.class).getPsProductUrl();

    /**
     * 通过明细查询询价详情
     *
     * @param id 报价明细id
     * @return
     */
    @Override
    public PublicInquiryItem findById(Long id, Long enUU) throws IllegalAccessException {
        if (StringUtils.isEmpty(enUU)) {
            throw new IllegalAccessException("无访问权限");
        }
        PublicInquiryItem item = infoDao.findOne(id);
        if (!enUU.equals(item.getVendUU())) {
            throw new IllegalAccessException("无法查询该条信息");
        }
        if (null != item.getProductId()) {
            Product product = productDao.findOne(item.getProductId());
            item.setProduct(product);
        } else {
            Product product = new Product();
            product.setCode(item.getProdCode());
            product.setSpec(item.getSpec());
            product.setTitle(item.getProdTitle());
            product.setCmpCode(item.getCmpCode());
            product.setBrand(item.getInbrand());
            if (null == product.getCode()) {
                product.setCode(item.getCmpCode());
            }
            item.setProduct(product);
        }
        return item;
    }

    /**
     * 供应商报价时上传附件信息
     *
     * @param attach
     * @return
     */
    @Override
    public Attach addAttaches(Attach attach) {
        return attachDao.save(attach);
    }

    /**
     * 供应商保存报价信息
     *
     * @param item 报价信息
     * @return
     * @author hejq
     * @date 2018-01-14 16:43
     */
    @Override
    public PublicInquiryItem add(PublicInquiryItem item) {
        PublicInquiryItem existInquiry = infoDao.findByVendUUAndSourceId(item.getVendUU(), item.getId());
        if (null != existInquiry) {
            throw new IllegalOperatorException("单据已报价，不能重复报价");
        }
        jdbcTemplate.update("insert into public$inquiryitems(id_id,id_number,id_currency,id_fromdate,id_taxrate,id_prid,id_venduu,"
                        + "id_venduseruu,id_sourceapp,id_status,id_sendstatus,id_sourceid,id_inid,id_replace_cmpcode,id_replace_spec,id_replace_brand,id_isreplace) values (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)",
                item.getId(), item.getNumber(), item.getCurrency(), item.getFromDate(),
                item.getTaxrate(), item.getProductId(), item.getVendUU(), item.getVendUserUU(),
                item.getSource(), item.getStatus(), item.getSendStatus(),
                item.getSourceId(), item.getInquiry().getId(),
                item.getReplaceCmpCode(), item.getReplaceSpec(), item.getReplaceBrand(), item.getIsReplace());
        //回复清单更新
        if(!CollectionUtils.isEmpty(item.getReplies())) {
            for (PublicInquiryReply reply1 : item.getReplies()) {
                jdbcTemplate.update("update public$inquiryreply set ir_idid = " + item.getId() + "where ir_id = " + reply1.getId());
            }
        }
        return infoDao.findOne(item.getId());
    }

    /**
     * 保存询价信息
     *
     * @param items 询价信息
     * @return
     * @author hejq
     * @date 2018-01-15 9:16
     */
    @Override
    public List<PublicInquiryItem> saveItems(List<PublicInquiryItem> items) {
        return infoDao.save(items);
    }

    /**
     * 保存询价信息
     *
     * @param inquiryItem 报价信息
     * @return
     */
    @Override
    public PublicInquiryItem save(PublicInquiryItem inquiryItem) {
        return infoDao.save(inquiryItem);
    }

    /**
     * 通过公共询价明细id查询报价信息
     *
     * @param id   公共询价明细id
     * @param enuu 供应商UU
     * @return
     */
    @Override
    public PublicInquiryItem findBySourceIdAndEnuu(Long id, Long enuu) {
        return infoDao.findByVendUUAndSourceId(enuu, id);
    }

    /**
     * 转成报价询价单
     *
     * @param inquiryItem 询价明细
     * @return
     */
    @Override
    public PublicInquiryItem saveItem(PublicInquiryItem inquiryItem) {
        if (StringUtils.isEmpty(inquiryItem.getVendUU()) || StringUtils.isEmpty(inquiryItem.getVendUserUU())) {
            throw new com.uas.ps.core.page.exception.IllegalOperatorException("企业UU或用户UU不能为空");
        }
        PublicInquiryItem existInquiry = infoDao.findByVendUUAndSourceId(inquiryItem.getVendUU(), inquiryItem.getId());
        if (null != existInquiry) {
            throw new IllegalOperatorException("该单据已报价，不能重复报价");
        }
        PublicInquiry inquiry = inquiryItem.getInquiry();
        if (null != inquiry) {
            if (inquiry.getEnUU() != null && inquiryItem.getVendUU().equals(inquiry.getEnUU())) {
                throw new IllegalOperatorException("不能对自己企业进行报价");
            }
            // 先判断客户询价单是否存在这张单据
            List<PublicInquiry> saleInquiries = inquiryDao.findByEnUUAndCode(inquiry.getEnUU(), inquiry.getCode());
            // 新增
            if (CollectionUtils.isEmpty(saleInquiries)) {
                inquiry.setSourceId(inquiry.getId());
                inquiry.setId(null);
                inquiry.setInquiryItems(null);
                if (!CollectionUtils.isEmpty(inquiry.getAttachs())) {
                    Set<Attach> attachs = new HashSet<Attach>();
                    for (com.uas.ps.inquiry.model.Attach attach : inquiry.getAttachs()) {
                        com.uas.ps.inquiry.model.Attach newAttach = new com.uas.ps.inquiry.model.Attach();
                        newAttach.setDate(new Date());
                        newAttach.setDescription(attach.getDescription());
                        newAttach.setName(attach.getName());
                        newAttach.setPath(attach.getPath());
                        newAttach.setRelatedKey(attach.getRelatedKey());
                        newAttach.setSize(attach.getSize());
                        attachs.add(newAttach);
                    }
                    inquiry.setAttachs(attachs);

                }
                if (null == inquiry.getEnName() && !StringUtils.isEmpty(inquiry.getEnUU())) {
                    Enterprise e = enterpriseDao.findOne(inquiry.getEnUU());
                    inquiry.setEnName(e.getEnName());
                }
                inquiry = inquiryDao.save(inquiry);

                PublicInquiryItem item = saveInquiryItems(inquiry, inquiryItem);
                // 报价成功后保存到个人物料库中
                if (null != item && !StringUtils.isEmpty(item.getInbrand()) && !StringUtils.isEmpty(item.getCmpCode())) {
                    saveInquiryItemProduct(item);
                }
                return item;
            } else {// 插入
                inquiry = saleInquiries.get(0);
                PublicInquiryItem item = saveInquiryItems(inquiry, inquiryItem);
                // 报价成功后保存到个人物料库中
                if (null != item && !StringUtils.isEmpty(item.getInbrand()) && !StringUtils.isEmpty(item.getCmpCode())) {
                    saveInquiryItemProduct(item);
                }
                return item;
            }
        }
        return null;
    }

    /**
     * 报价成功后保存到个人物料库中
     * @param inquiryItem
     */
    private void saveInquiryItemProduct(PublicInquiryItem inquiryItem) {
        Product product = new Product();
        // 需要哪些字段
        product.setEnUU(inquiryItem.getVendUU());
        product.setUserUU(inquiryItem.getVendUserUU());
        product.setCmpCode(inquiryItem.getCmpCode());
        product.setBrand(inquiryItem.getInbrand());
        product.setpCmpCode(inquiryItem.getCmpCode());
        product.setpBrandEn(inquiryItem.getInbrand());
        product.setStandard((short) 0);
        product.setCreateTime(new Date());
        product.setSourceApp(inquiryItem.getQutoApp());
        product.setSpec(StringUtils.isEmpty(inquiryItem.getSpec()) ? inquiryItem.getCmpCode() : inquiryItem.getSpec());
        product.setKind(inquiryItem.getProdTitle());
        product.setSpec((inquiryItem.getSpec() == null || "".equals(inquiryItem.getSpec())) ? inquiryItem.getCmpCode() : inquiryItem.getSpec());
        // 调用公共物料服务的接口
        try {
            String url = PS_PRODUCT_URL + "/product/save/inquiry";
            // 如果是替代料报价，则增加参数
            if (Constant.YES == inquiryItem.getIsReplace() && !StringUtils.isEmpty(inquiryItem.getReplaceCmpCode())
                    && !StringUtils.isEmpty(inquiryItem.getReplaceBrand())) {
                url = url + "?replaceCmpCode=" + URLEncoder.encode(inquiryItem.getReplaceCmpCode(), "UTF-8") + "&replaceBrand=" + URLEncoder.encode(inquiryItem.getReplaceBrand(), "UTF-8");
            }
            long start = System.currentTimeMillis();
            String res = HttpUtil.doPost(url, FlexJsonUtils.toJsonDeep(product));
            log.info("/product/save/inquiry 耗时：{}", (System.currentTimeMillis() - start));
            JSONObject result = JSONObject.parseObject(res);
            Boolean success = (Boolean) result.get("success");
            if (success) {
                log.info("报价成功后保存到个人物料库，保存报价id【{}】到个人物料库成功，保存成功，UserUU:{}，EnUU:{}，保存询价的应用:{}", inquiryItem.getId(), product.getUserUU(), product.getEnUU(), inquiryItem.getSource());
            } else {
                log.info("报价成功后保存到个人物料库，保存报价id【{}】到个人物料库失败{}，UserUU:{}，EnUU:{}，保存询价的应用:{}", inquiryItem.getId(), result.get("message").toString(), product.getUserUU(), product.getEnUU(), inquiryItem.getSource());
            }
        } catch (Exception e) {
            log.info("/product/save/inquiry 报价成功后保存到个人物料库中 出错：{}", e.getMessage());
            e.printStackTrace();
            String reason = e.getMessage().length() > 999 ? e.getMessage().substring(0, 999).concat("") : e.getMessage();
            log.info("报价成功后保存到个人物料库，保存报价id【{}】到个人物料库失败，原因：{}，UserUU:{}, EnUU:{}，保存询价的应用:{}", inquiryItem.getId(), reason, product.getUserUU(), product.getEnUU(), inquiryItem.getSource());
        }
    }

    /**
     * 保存明细
     * @param inquiry 报价主表信息
     * @param inquiryItem 报价明细信息
     * @return 报价单
     */
    private PublicInquiryItem saveInquiryItems(PublicInquiry inquiry, PublicInquiryItem inquiryItem) {
        long start = System.currentTimeMillis();
        // 设置报价明细信息
        setPublicInquiryItemWithInquiry(inquiry, inquiryItem);

        inquiryItem = infoDao.save(inquiryItem);
        log.info("save publicInquiryItem耗时:" + (System.currentTimeMillis() - start));
        start = System.currentTimeMillis();

        // 更新原表的报价条数
        purcInquiryItemDao.updateAmount(inquiryItem.getSourceId(), 1);
        System.out.println("更新报价条数耗时:" + (System.currentTimeMillis() - start));
        start = System.currentTimeMillis();

        // 更新推荐表中的相关信息
        remindDao.updateStatus(inquiryItem.getId(), inquiryItem.getSourceId(), Status.SUBMITTED.value(), inquiryItem.getVendUU());
        System.out.println("更新个人推荐耗时:" + (System.currentTimeMillis() - start));
        start = System.currentTimeMillis();

        // 更新企业推荐中的相关信息
        inquiryEnRemindDao.updateStatus(inquiryItem.getId(), inquiryItem.getSourceId(), Status.SUBMITTED.value(), inquiryItem.getVendUU());
        System.out.println("更新企业推荐耗时:" + (System.currentTimeMillis() - start));
        return inquiryItem;
    }

    /**
     * 设置报价明细信息
     * @param inquiry  报价单
     * @param inquiryItem  报价明细
     */
    private void setPublicInquiryItemWithInquiry(PublicInquiry inquiry, PublicInquiryItem inquiryItem) {
        // 计算合计价格 dongbw  2018年9月5日 15:08:24  bom求购明细价格分析需要
        Double sumPrice = 0d;
        Double needQty = null != inquiryItem.getNeedquantity() ? inquiryItem.getNeedquantity() : 1;
        Double maxQty = 0d;
        Double maxPrice = 0d;
        List<PublicInquiryReply> replies = inquiryItem.getReplies();
        for (int i = 0; i < replies.size(); i++) {
            maxQty = replies.get(i).getLapQty();
            maxPrice = replies.get(i).getPrice();
            if (needQty < maxQty) {
                if (i == 0) {
                    sumPrice = replies.get(i).getPrice() * needQty;
                } else {
                    sumPrice = replies.get(i - 1).getPrice() * needQty;
                }
                break;
            }
        }
        if (needQty >= maxQty) {
            sumPrice = maxPrice * needQty;
        }
        inquiryItem.setSumPrice(sumPrice);
        int number = 1;
        if (null != inquiryItem.getInquiry() && null != inquiryItem.getInquiry().getId()) {
            number = infoDao.countByInquiryId(inquiryItem.getInquiry().getId()) + 1;
        }
        Enterprise enterprise = enterpriseDao.findOne(inquiryItem.getVendUU());
        if (null != inquiryItem.getProduct()) {
            inquiryItem.setProdTitle(inquiryItem.getProduct().getTitle());
            inquiryItem.setSpec(inquiryItem.getProduct().getSpec());
            inquiryItem.setInbrand(inquiryItem.getProduct().getBrand());
            inquiryItem.setProdCode(inquiryItem.getProduct().getCode());
        }
        inquiryItem.setVendName(enterprise.getEnName());
        if (StringUtils.isEmpty(inquiryItem.getEndDate())) {
            inquiryItem.setEndDate(inquiry.getEndDate());
        }
        inquiryItem.setBusinessCode(enterprise.getBusinesscode());
        inquiryItem.setDate(inquiry.getDate());
        inquiryItem.setOfferTime(new Date(System.currentTimeMillis()));
        inquiryItem.setInquiry(inquiry);
        inquiryItem.setNumber((short) number);
        inquiryItem.setSendStatus((short) Status.NOT_UPLOAD.value());
        inquiryItem.setSourceId(inquiryItem.getId());
        inquiryItem.setId(null);
        inquiryItem.setStatus((short) Status.REPLIED.value());
        inquiryItem.setBackStatus((short) Status.NOT_UPLOAD.value());
        inquiryItem.setReplySendStatus((short) Status.NOT_UPLOAD.value());
        // 将采纳状态设置成申请状态，方便过滤
        inquiryItem.setDecideStatus((short) Status.UNAUDIT.value());
    }

    /**
     * 转报价后进行物料同步
     *
     * @param publicInquiryItems
     */
    private void syscProduct(List<PublicInquiryItem> publicInquiryItems) throws Exception {
        for (PublicInquiryItem item : publicInquiryItems) {
            List<Product> products = new ArrayList<>();
            List<Product> productList = productDao.findByEnUUAndCmpCodeAndBrand(item.getVendUU(), item.getCmpCode(), item.getInbrand());
            if (CollectionUtils.isEmpty(productList)) {
                Product product = new Product();
                SimpleDateFormat sdf = new SimpleDateFormat("yyyymmddsss");
                product.setCode(sdf.format(new Date()) + item.getCmpCode());
                product.setBrand(item.getInbrand());
                product.setCmpCode(item.getCmpCode());
                product.setTitle(item.getProdTitle());
                product.setSpec(item.getSpec());
                product.setEnUU(item.getVendUU());
                product.setUserUU(item.getVendUserUU());
                product.setIsSale(Constant.YES);
                products.add(product);
            }
            if (!CollectionUtils.isEmpty(products)) {
                long start = System.currentTimeMillis();
                HttpUtil.doPost(PS_PRODUCT_URL + "product/update/b2b", FlexJsonUtils.toJsonDeep(products));
                log.info("product/update/b2b 耗时：{}，物料数：{}", (System.currentTimeMillis() - start), products.size());
            }
        }
    }

    /**
     * 通过id查询公共询价详情
     *
     * @param id   主键id
     * @param enuu 企业uu
     * @return
     */
    @Override
    public ModelMap findByIdAndEnuu(Long id, Long enuu) {
        ModelMap map = new ModelMap();
        PurcInquiryItemInfo inquiryItem = inquiryItemInfoDao.findOne(id);
        if (null != inquiryItem) {
            if (null == inquiryItem.getProductId()) {
                Product product = new Product();
                product.setTitle(inquiryItem.getProdTitle());
                product.setCmpCode(inquiryItem.getCmpCode());
                product.setBrand(inquiryItem.getInbrand());
                product.setSpec(inquiryItem.getSpec());
                product.setCode(inquiryItem.getProdCode());
                if (product.getCode() == null) {
                    product.setCode(inquiryItem.getCmpCode());
                }
                inquiryItem.setProduct(product);
            }
            map.put("inquiryItem", inquiryItem);
        }
        PublicInquiryItem item = infoDao.findByVendUUAndSourceId(enuu, id);
        if (null != item) {
            map.put("idid", item.getId());
        }
        return map;
    }

    /**
     * 移动端转报价的功能
     *
     * @param id      公共询价明细id
     * @param enuu    报价企业UU号
     * @param replies 报价信息
     * @param leadtime 交货周期
     * @param minOrderQty 最小订购
     * @param minPackQty 最小包装
     * @param useruu 个人UU
     * @param currency 币别
     * @param taxrate 税率
     */
    @Override
    public ModelMap quoteInquiry(Long id, Long enuu, Long useruu, List<PublicInquiryReply> replies, Long leadtime, Double minPackQty, Double minOrderQty,  String currency, Float taxrate) throws Exception {
        ModelMap map = new ModelMap();
        PublicInquiryItem existInquiry = infoDao.findByVendUUAndSourceId(enuu, id);
        if (null != existInquiry) {
            throw new IllegalOperatorException("该单据已报价，不能重复报价");
        }
        PurcInquiryItemInfo item = inquiryItemInfoDao.findOne(id);
        if (item != null && item.getInquiry() != null && item.getInquiry().getEnUU().equals(enuu)) {
            throw new IllegalOperatorException("不能对自己单据进行报价");
        }
        PublicInquiry inquiry = new PublicInquiry();
        List<PublicInquiry> saleInquiries = inquiryDao.findByEnUUAndCode(item.getInquiry().getEnUU(), item.getInquiry().getCode());
        PublicInquiryItem inquiryItem = new PublicInquiryItem(item);
        inquiryItem.setMinOrderQty(minOrderQty);
        inquiryItem.setMinPackQty(minPackQty);
        inquiryItem.setLeadtime(leadtime);
        inquiryItem.setQutoApp(InquirySource.UU.name());
        inquiryItem.setReplies(replies);
        inquiryItem.setCurrency(currency);
        inquiryItem.setTaxrate(taxrate);

        // 先判断客户询价单是否存在这张单据
        if (CollectionUtils.isEmpty(saleInquiries)) {
            inquiry = new PublicInquiry(item.getInquiry());
            if (null == inquiry.getEnName()) {
                Enterprise e = enterpriseDao.findOne(inquiry.getEnUU());
                inquiry.setEnName(e.getEnName());
            }
            inquiry = inquiryDao.save(inquiry);
            inquiryItem.setVendUU(enuu);
            inquiryItem.setVendUserUU(useruu);
            inquiryItem = saveInquiryItems(inquiry, inquiryItem);
            if (null != inquiryItem.getId()) {
                map.put("content", inquiryItem.getId());
            } else {
                map.put("content", false);
            }
            return map;
        } else {// 插入
            inquiry = saleInquiries.get(0);
            PublicInquiryItem purcItem = infoDao.findByVendUUAndSourceId(inquiryItem.getVendUU(), inquiryItem.getSourceId());
            if (purcItem != null) {
                throw new Exception("询价单已存在，不允许再次报价");
            } else {
                inquiryItem.setInquiry(inquiry);
                // 这里设置id是为了后面赋值来源id
                inquiryItem.setId(id);
                inquiryItem.setOfferTime(new Date(System.currentTimeMillis()));
                inquiryItem.setVendUU(enuu);
                inquiryItem.setVendUserUU(useruu);
                inquiryItem = saveInquiryItems(inquiry, inquiryItem);
                if (null != inquiryItem.getId()) {
                    map.put("content", inquiryItem.getId());
                } else {
                    map.put("content", false);
                }
                return map;
            }
        }
    }

    /**
     * 通过分页信息查询询价推荐信息
     *
     * @param pageInfo     分页信息
     * @param filter 过滤条件
     * @return
     */
    @Override
    public IPage<InquiryRemind> findRemindByPageInfo(final PageInfo pageInfo, final SearchFilter filter) {
        Sort sort = new Sort(Sort.Direction.DESC, "date");
        if (pageInfo.getOffset() == 0) {
            pageInfo.setOffset(pageInfo.getPageSize() * (pageInfo.getPageNumber() - 1));
        }
        pageInfo.setSort(sort);
        if (null != filter) {
            if (StringUtils.hasText(filter.getKeyword())) {
                SimpleExpression title = new SimpleExpression("prodTitle", filter.getKeyword(), CriterionExpression.Operator.LIKE, true);
                SimpleExpression cmpCode = new SimpleExpression("cmpCode", filter.getKeyword(), CriterionExpression.Operator.LIKE, true);
                SimpleExpression spec = new SimpleExpression("spec", filter.getKeyword(), CriterionExpression.Operator.LIKE, true);
                SimpleExpression brand = new SimpleExpression("inbrand", filter.getKeyword(), CriterionExpression.Operator.LIKE, true);
                SimpleExpression enName = new SimpleExpression("enName", filter.getKeyword(), CriterionExpression.Operator.LIKE, true);
                SimpleExpression userName = new SimpleExpression("userName", filter.getKeyword(), CriterionExpression.Operator.LIKE, true);
                SimpleExpression[] simpleExpressions = new SimpleExpression[]{title, brand, cmpCode, spec, enName, userName};
                LogicalExpression logical = new LogicalExpression(simpleExpressions, CriterionExpression.Operator.OR);
                pageInfo.expression(logical);
            }
            if (filter.getFromDate() != null) {
                pageInfo.expression(PredicateUtils.gte("reDate", new Date(filter.getFromDate()), false));
            }
            if (filter.getEndDate() != null) {
                pageInfo.expression(PredicateUtils.lte("reDate", new Date(filter.getEndDate()), false));
            }
            // 默认返回可报价
            if (filter.getEnableOffer() != null && filter.getEnableOffer() == 0) {
                pageInfo.expression(PredicateUtils.lte("endDate", new Date(), false));
            } else {
                pageInfo.expression(PredicateUtils.gt("endDate", new Date(), false));
            }

            // 默认返回待报价
            if (filter.getQuoted() != null && filter.getQuoted() == 1) {
                pageInfo.expression(PredicateUtils.isNotNull("newId"));
            } else {
                pageInfo.expression(PredicateUtils.isNull("newId"));
            }
        }
        Page reminds = remindDao.findAll(new Specification<InquiryRemind>() {
            @Override
            public Predicate toPredicate(Root<InquiryRemind> root, CriteriaQuery<?> query, CriteriaBuilder builder) {
                query.where(pageInfo.getPredicates(root, query, builder));
                return null;
            }
        }, pageInfo);
        return covert(reminds);
    }

    /**
     * 将查询的分页信息封装成需要的数据
     *
     * @param reminds 推荐询价信息
     * @return
     */
    private IPage<InquiryRemind> covert(Page reminds) {
        IPage<InquiryRemind> remindIPage = IPageUtils.covertIgnoreContent(reminds);
        if (!CollectionUtils.isEmpty(reminds.getContent())) {
            List<InquiryRemind> remindList = reminds.getContent();
            for (InquiryRemind remind : remindList) {
                PublicInquiryItem item = infoDao.findByVendUUAndSourceId(remind.getVendUU(), remind.getItemId());
                PurcInquiryItem item1 = purcInquiryItemDao.findOne(remind.getItemId());
                if (null != item) {
                    remind.setNewId(item.getId());
                    remind.setQuotation(item);
                    remind.setStatus(Status.SUBMITTED.value());
                    if (null != item.getAgreed()) {
                        if (item.getAgreed().equals(Constant.YES)) {
                            remind.setStatus(Status.ALLOW.value());
                        } else if (item.getAgreed().equals(Constant.NO)) {
                            remind.setStatus(Status.NOTALLOW.value());
                        }
                    }
                } else if (null == item && null != item1 && null != item1.getOverdue()) {
                    if (item1.getOverdue().equals(Constant.YES)) {
                        remind.setStatus(Status.FINISH.value());
                    }
                }
            }
            remindIPage.setContent(remindList);
        }
        return remindIPage;
    }

    /**
     * 分页查询企业商机
     * @param pageInfo
     * @param filter
     * @return
     */
    @Override
    public IPage<InquiryEnRemind> findEnRemindByPageInfo(final PageInfo pageInfo, final SearchFilter filter) {
        Sort sort = new Sort(Sort.Direction.DESC, "date");
        if (pageInfo.getOffset() == 0) {
            pageInfo.setOffset(pageInfo.getPageSize() * (pageInfo.getPageNumber() - 1));
        }
        pageInfo.setSort(sort);
        Integer isAll = null;
        if (null != filter) {
            if (StringUtils.hasText(filter.getKeyword())) {
                SimpleExpression title = new SimpleExpression("prodTitle", filter.getKeyword(), CriterionExpression.Operator.LIKE, true);
                SimpleExpression cmpCode = new SimpleExpression("cmpCode", filter.getKeyword(), CriterionExpression.Operator.LIKE, true);
                SimpleExpression spec = new SimpleExpression("spec", filter.getKeyword(), CriterionExpression.Operator.LIKE, true);
                SimpleExpression brand = new SimpleExpression("inbrand", filter.getKeyword(), CriterionExpression.Operator.LIKE, true);
                SimpleExpression enName = new SimpleExpression("enName", filter.getKeyword(), CriterionExpression.Operator.LIKE, true);
                SimpleExpression userName = new SimpleExpression("userName", filter.getKeyword(), CriterionExpression.Operator.LIKE, true);
                SimpleExpression[] simpleExpressions = new SimpleExpression[]{title, brand, cmpCode, spec, enName, userName};
                LogicalExpression logical = new LogicalExpression(simpleExpressions, CriterionExpression.Operator.OR);
                pageInfo.expression(logical);
            }
            if (filter.getFromDate() != null) {
                pageInfo.expression(PredicateUtils.gte("reDate", new Date(filter.getFromDate()), false));
            }
            if (filter.getEndDate() != null) {
                pageInfo.expression(PredicateUtils.lte("reDate", new Date(filter.getEndDate()), false));
            }
            if (!StringUtils.isEmpty(filter.getIsAll()) && filter.getIsAll() == 0) {
                pageInfo.filter("remindStatus", 0);
                isAll = filter.getIsAll();
            }
            // 默认返回可报价
            if (filter.getEnableOffer() != null && filter.getEnableOffer() == 0) {
                pageInfo.expression(PredicateUtils.lte("endDate", new Date(), false));
            } else {
                pageInfo.expression(PredicateUtils.gt("endDate", new Date(), false));
            }

            // 默认返回待报价
            if (filter.getQuoted() != null && filter.getQuoted() == 1) {
                pageInfo.expression(PredicateUtils.isNotNull("newId"));
            } else {
                pageInfo.expression(PredicateUtils.isNull("newId"));
            }
        }
        Page reminds = inquiryEnRemindDao.findAll(new Specification<InquiryEnRemind>() {
            @Override
            public Predicate toPredicate(Root<InquiryEnRemind> root, CriteriaQuery<?> query, CriteriaBuilder builder) {
                query.where(pageInfo.getPredicates(root, query, builder));
                return null;
            }
        }, pageInfo);
        return covertEnRemindIPage(reminds, isAll);
    }

    @Override
    public ModelMap allotEnRemind(Long userUU, Long enRemindId) {
        if (StringUtils.isEmpty(userUU) && StringUtils.isEmpty(enRemindId)) {
            throw new IllegalAccessError("缺少参数");
        }
        ModelMap result = new ModelMap();
        // 从公司商机分配到个人
        InquiryEnRemind inquiryEnRemind = inquiryEnRemindDao.findOne(enRemindId);
        if (!StringUtils.isEmpty(inquiryEnRemind)) {
            InquiryRemind inquiryRemind = enRemindConvertRemind(inquiryEnRemind);
            inquiryRemind.setVendUserUU(userUU);
            remindDao.save(inquiryRemind);
            inquiryEnRemind.setRemindStatus(1);
            inquiryEnRemindDao.save(inquiryEnRemind);
            result.put("success", true);
        } else {
            result.put("success", false);
            result.put("message", "企业商机不存在");
        }
        return result;
    }

    @Override
    public ModelMap allotInquiry(Long userUU, Long enUU, Long itemId) {
        if (StringUtils.isEmpty(userUU) && StringUtils.isEmpty(enUU) && StringUtils.isEmpty(itemId)) {
            throw new IllegalAccessError("缺少参数");
        }
        ModelMap result = new ModelMap();
        // 判断当前商机是否在企业商机
        InquiryEnRemind oldEnRemind = inquiryEnRemindDao.findByItemIdAndEnUU(itemId, enUU);
        if (StringUtils.isEmpty(oldEnRemind)) { // 不存在
            PurcInquiryItem item = purcInquiryItemDao.findOne(itemId);
            Enterprise enterprise = enterpriseDao.findOne(enUU);
            InquiryEnRemind enRemind = itemConvertEnRemind(item);
            enRemind.setEnName(enterprise.getEnName());
            enRemind.setVendUU(enUU);
            enRemind.setRemindStatus(1);
            inquiryEnRemindDao.save(enRemind);
            // 再分配给个人
            InquiryRemind remind = enRemindConvertRemind(enRemind);
            remind.setVendUserUU(userUU);
            remindDao.save(remind);
            result.put("success", true);
        } else { // 存在
            // 判断个人商机是否存在
            List<InquiryRemind> oldRemind = remindDao.findByVendUserUUAndVendUUAndItemId(userUU, enUU, itemId);
            if (CollectionUtils.isEmpty(oldRemind)) {
                InquiryRemind remind = enRemindConvertRemind(oldEnRemind);
                oldEnRemind.setRemindStatus(1);
                inquiryEnRemindDao.save(oldEnRemind);
                remind.setVendUserUU(userUU);
                remindDao.save(remind);
                result.put("success", true);
            } else {
                result.put("success", false);
                result.put("message", "已存在当前用户的个人商机中");
            }
        }
        return result;
    }

    @Override
    public ModelMap saveQuote(PublicInquiryItem publicInquiryItem) {
        ModelMap result = new ModelMap();
        result.put("success", false);
        // 验证报价单信息
        if (validatePublicItem(publicInquiryItem, result)) {
            return result;
        }
        // 可能由于直接使用询价明细实体转换报价，而未清除id，所以这里参数中的id可能是询价明细id
        Long itemId = null != publicInquiryItem.getId() ? publicInquiryItem.getId() : publicInquiryItem.getSourceId();
        if (null == itemId) {
            result.put("message", "未传入有效的询价明细id");
            return result;
        }
        // 是否为自己的询价 并查询公共询价的信息
        PurcInquiryItemInfo item = inquiryItemInfoDao.findOne(itemId);
        // 验证询价明细信息
        if (validatePurcInquiryItem(publicInquiryItem, result, item)) {
            return result;
        }
        // 验证是否重复
        if (validateRepeat(publicInquiryItem, result, itemId)) {
            return result;
        }

        // 拼接报价单主表
        PublicInquiry inquiry = getPublicInquiry(item);

        // 拼接报价单明细
        PublicInquiryItem inquiryItem = getPublicInquiryItem(publicInquiryItem, item);
        inquiryItem.setInquiry(inquiry);
        // 调用回复询价接口
        return replyInquiry(result, inquiry, inquiryItem);
    }

    private boolean validateRepeat(PublicInquiryItem publicInquiryItem, ModelMap result, Long itemId) {
        PublicInquiryItem existInquiry = infoDao.findByVendUUAndSourceId(publicInquiryItem.getVendUU(), itemId);
        // 是否重复报价
        if (null != existInquiry) {
            result.put("message", "该单据已报价，不能重复报价");
            return true;
        }
        return false;
    }

    private boolean validatePurcInquiryItem(PublicInquiryItem publicInquiryItem, ModelMap result, PurcInquiryItemInfo item) {
        if (null == item) {
            result.put("message", "找不到当前公共询价");
            return true;
        }
        if (item.getInquiry() != null && publicInquiryItem.getVendUU().equals(item.getInquiry().getEnUU())) {
            result.put("message", "不能对自己单据进行报价");
            return true;
        }
        return false;
    }

    private boolean validatePublicItem(PublicInquiryItem publicInquiryItem, ModelMap result) {
        // 判断报价人信息
        if (StringUtils.isEmpty(publicInquiryItem) || StringUtils.isEmpty(publicInquiryItem.getVendUU()) || StringUtils.isEmpty(publicInquiryItem.getVendUserUU())) {
            result.put("message", "报价人信息不能为空");
            return true;
        }
        if (CollectionUtils.isEmpty(publicInquiryItem.getReplies())) {
            result.put("message", "未找到报价信息");
            return true;
        }
        return false;
    }

    /**
     * 报价接口调用
     * @param result  ModelMap
     * @param inquiry  报价明细
     * @param inquiryItem  报价明细
     * @return
     */
    private ModelMap replyInquiry(ModelMap result, PublicInquiry inquiry, PublicInquiryItem inquiryItem) {
        try {
            long start = System.currentTimeMillis();
            inquiryItem = saveInquiryItems(inquiry, inquiryItem);
            System.out.println("报价总耗时:" + (System.currentTimeMillis() - start));
            start = System.currentTimeMillis();
            // 报价成功后保存到个人物料库中
            if (null != inquiryItem && !StringUtils.isEmpty(inquiryItem.getInbrand()) && !StringUtils.isEmpty(inquiryItem.getCmpCode())) {
                saveInquiryItemProduct(inquiryItem);
                System.out.println("报价成功，添加个人库耗时:" + (System.currentTimeMillis() - start));
            }
            result.put("success", true);
            result.put("content", inquiryItem);
            return result;
        } catch (Exception e) {
            e.printStackTrace();
            result.put("message", e.getMessage());
            return result;
        }
    }

    /**
     * 拼接报价单主表
     * @param item  询价明细
     * @return 报价主表
     */
    private PublicInquiry getPublicInquiry(PurcInquiryItemInfo item) {
        // 是否已经已经转报价
        List<PublicInquiry> saleInquiries = inquiryDao.findByEnUUAndCode(item.getInquiry().getEnUU(), item.getInquiry().getCode());
        PublicInquiry inquiry = new PublicInquiry();
        if (CollectionUtils.isEmpty(saleInquiries)) {
            // 附件
            if (!CollectionUtils.isEmpty(inquiry.getAttachs())) {
                Set<Attach> attachs = new HashSet<Attach>();
                for (Attach attach : inquiry.getAttachs()) {
                    Attach newAttach = new Attach();
                    newAttach.setDate(new Date());
                    newAttach.setDescription(attach.getDescription());
                    newAttach.setName(attach.getName());
                    newAttach.setPath(attach.getPath());
                    newAttach.setRelatedKey(attach.getRelatedKey());
                    newAttach.setSize(attach.getSize());
                    attachs.add(newAttach);
                }
                inquiry.setAttachs(attachs);

            }
            // 转报价后 主表信息保存
            inquiry = new PublicInquiry(item.getInquiry());
            if (null == inquiry.getEnName() && null != inquiry.getEnUU()) {
                Enterprise e = enterpriseDao.findOne(inquiry.getEnUU());
                inquiry.setEnName(e.getEnName());
            }
            inquiry = inquiryDao.save(inquiry);
        } else { // 已经存在
            inquiry = saleInquiries.get(0);
        }
        return inquiry;
    }

    /**
     * 拼接报价明细
     * @param publicInquiryItem 报价明细参数
     * @param item  询价明细
     * @return  报价明细
     */
    private PublicInquiryItem getPublicInquiryItem(PublicInquiryItem publicInquiryItem, PurcInquiryItemInfo item) {
        // 报价明细
        PublicInquiryItem inquiryItem = new PublicInquiryItem(item);
        inquiryItem.setMinOrderQty(publicInquiryItem.getMinOrderQty());
        inquiryItem.setMinPackQty(publicInquiryItem.getMinPackQty());
        inquiryItem.setLeadtime(publicInquiryItem.getLeadtime());
        inquiryItem.setQutoApp(publicInquiryItem.getQutoApp());
        inquiryItem.setReplies(publicInquiryItem.getReplies());
        inquiryItem.setCurrency(publicInquiryItem.getCurrency());
        inquiryItem.setTaxrate(publicInquiryItem.getTaxrate());
        inquiryItem.setVendUU(publicInquiryItem.getVendUU());
        inquiryItem.setVendUserUU(publicInquiryItem.getVendUserUU());
        inquiryItem.setReplaceCmpCode(publicInquiryItem.getReplaceCmpCode());
        inquiryItem.setReplaceBrand(publicInquiryItem.getReplaceBrand());
        inquiryItem.setReplaceSpec(publicInquiryItem.getReplaceSpec());
        inquiryItem.setIsReplace(publicInquiryItem.getIsReplace());
        return inquiryItem;
    }

    private InquiryEnRemind itemConvertEnRemind(PurcInquiryItem item) {
        InquiryEnRemind remind = new InquiryEnRemind();
        //remind.setEnName(enterprise.getEnName());
        remind.setEnUU(item.getInquiry().getEnUU());
        remind.setUserName(item.getUserName());
        remind.setUserTel(item.getUserTel());
        remind.setCode(item.getInquiry().getCode());
        remind.setBrand(item.getInbrand());
        remind.setInbrand(item.getInbrand());
        remind.setCmpCode(item.getCmpCode());
        remind.setSpec(item.getSpec());
        remind.setProdTitle(item.getProdTitle());
        remind.setInid(item.getInquiry().getId());
        if (item.getProduct() != null) {
            remind.setBrand(item.getProduct().getBrand());
            remind.setCmpCode(item.getProduct().getCmpCode());
            remind.setSpec(item.getProduct().getSpec());
            remind.setProdTitle(item.getProduct().getTitle());
        }
        remind.setItemId(item.getId());
        remind.setNeedQty(item.getNeedquantity());
        remind.setReDate(item.getInquiry().getDate());
        remind.setEndDate(item.getInquiry().getEndDate());
        //remind.setVendUU(u.getEnuu());
        remind.setDate(new Date(System.currentTimeMillis()));
        remind.setStatus(Status.INPUTTING.value());
        return remind;
    }

    private InquiryRemind enRemindConvertRemind(InquiryEnRemind inquiryEnRemind) {
        InquiryRemind remind = new InquiryRemind();
        remind.setEnName(inquiryEnRemind.getEnName());
        remind.setEnUU(inquiryEnRemind.getEnUU());
        remind.setUserName(inquiryEnRemind.getUserName());
        remind.setUserTel(inquiryEnRemind.getUserTel());
        remind.setCode(inquiryEnRemind.getCode());
        remind.setBrand(inquiryEnRemind.getBrand());
        remind.setInbrand(inquiryEnRemind.getInbrand());
        remind.setCmpCode(inquiryEnRemind.getCmpCode());
        remind.setSpec(inquiryEnRemind.getSpec());
        remind.setProdTitle(inquiryEnRemind.getProdTitle());
        remind.setInid(inquiryEnRemind.getInid());
        remind.setItemId(inquiryEnRemind.getItemId());
        remind.setNeedQty(inquiryEnRemind.getNeedQty());
        remind.setReDate(inquiryEnRemind.getDate());
        remind.setEndDate(inquiryEnRemind.getEndDate());
        remind.setVendUU(inquiryEnRemind.getVendUU());
        remind.setDate(new Date(System.currentTimeMillis()));
        remind.setStatus(Status.INPUTTING.value());
        return remind;
    }

    /**
     * 将查询的分页信息封装成需要的数据
     *
     * @param enReminds 推荐询价信息
     * @return
     */
    private IPage<InquiryEnRemind> covertEnRemindIPage(Page enReminds, Integer isAll) {
        IPage<InquiryEnRemind> remindIPage = IPageUtils.covertIgnoreContent(enReminds);
        if (!CollectionUtils.isEmpty(enReminds.getContent())) {
            List<InquiryEnRemind> remindList = enReminds.getContent();
            for (InquiryEnRemind remind : remindList) {
                // 通过推荐的明细id 查询明细详细数据
                PublicInquiryItem item = infoDao.findByVendUUAndSourceId(remind.getVendUU(), remind.getItemId());
                PurcInquiryItem item1 = purcInquiryItemDao.findOne(remind.getItemId());
                if (null != item) {
                    remind.setNewId(item.getId());
                    remind.setQuotation(item);
                    remind.setStatus(Status.SUBMITTED.value());
                    if (null != item.getAgreed()) {
                        if (item.getAgreed().equals(Constant.YES)) {
                            remind.setStatus(Status.ALLOW.value());
                        } else if (item.getAgreed().equals(Constant.NO)) {
                            remind.setStatus(Status.NOTALLOW.value());
                        }
                    }
                } else if (null == item && null != item1 && null != item1.getOverdue()) {
                    if (item1.getOverdue().equals(Constant.YES)) {
                        remind.setStatus(Status.FINISH.value());
                    }
                }
                if (!StringUtils.isEmpty(isAll) && isAll == 0) {
                    remind.setInquiry(null);
                    remind.setQuotation(null);
                }
            }
            remindIPage.setContent(remindList);
        }
        return remindIPage;
    }
}
