package com.uas.ps.inquiry.controller;

import com.alibaba.fastjson.JSON;
import com.uas.ps.inquiry.BaseJunitTest;
import com.uas.ps.inquiry.entity.BatchInProduct;
import com.uas.ps.inquiry.entity.BatchInquiry;
import com.uas.ps.inquiry.entity.Inquiry;
import com.uas.ps.inquiry.entity.InquiryDecide;
import com.uas.ps.inquiry.util.FlexJsonUtils;
import com.uas.ps.inquiry.util.HttpUtil;
import org.junit.Test;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

/**
 * @author dongbw
 * @since 2018年9月27日
 */

public class ErpDownloadControllerTest extends BaseJunitTest {

    /**
     * localhost
     */
    private static final String localhost = "http://10.1.51.82:24002";

    /**
     * dev
     */
    private static final String dev_url = "http://192.168.253.12:24000";

    /**
     * userUU
     */
    private static final Long userUU = 1000002503L;

    /**
     * enUU
     */
    private static final Long enUU = 10041166L;

    /**
     * /
     * @throws Exception
     */
    @Test
    public void testSaveInquiries() throws Exception {
        HashMap<String, Object> params = new HashMap<>();
        List<BatchInquiry> inquiries = new ArrayList<>();
        BatchInquiry batchInquiry = new BatchInquiry();
        batchInquiry.setBi_code("MALL201809270001");
        batchInquiry.setBi_date(new Date(System.currentTimeMillis()));
        batchInquiry.setBi_enddate(new Date(System.currentTimeMillis() + 1000 * 60 * 60 * 24 * 2));
        batchInquiry.setBi_id(1L);
        batchInquiry.setBi_kind("inquiryType");
        batchInquiry.setBi_recorder("dongbw");
        batchInquiry.setBi_recorduu(userUU);
        batchInquiry.setBi_remark("111");
        List<BatchInProduct> inProducts = new ArrayList<>();

        BatchInProduct inProduct = new BatchInProduct();
        inProduct.setBip_biid(1L);
        inProduct.setBip_currency("RMB");
        inProduct.setBip_detno((short) 1);
        inProduct.setBip_id(1L);
        inProduct.setBip_mobile("15671616315");
        inProduct.setBip_prodcode("PROD001");
        inProduct.setPr_brand("Rohm");
        inProduct.setPr_detail("名称");
        inProduct.setPr_spec("规格");
        inProduct.setYcgqty(1000d);
        inProducts.add(inProduct);
        batchInquiry.setInProducts(inProducts);
        inquiries.add(batchInquiry);

        params.put("inquiries", inquiries);
        params.put("enuu", enUU);
        params.put("address", "111");
        String res = HttpUtil.doPost(localhost+ "/inquiry/public", FlexJsonUtils.toJsonDeep(params));
        System.out.println(JSON.toJSON(res));
    }

    /**
     * /check
     * @throws Exception
     */
    @Test
    public void testUpdateInquiryStatus() throws Exception {
        HashMap<String, Object> params = new HashMap<>();
        List<BatchInquiry> inquiries = new ArrayList<>();
        BatchInquiry batchInquiry = new BatchInquiry();
        batchInquiry.setBi_code("MALL201809270001");
        batchInquiry.setBi_date(new Date(System.currentTimeMillis()));
        batchInquiry.setBi_enddate(new Date(System.currentTimeMillis() + 1000 * 60 * 60 * 24 * 2));
        batchInquiry.setBi_id(1L);
        batchInquiry.setBi_kind("inquiryType");
        batchInquiry.setBi_recorder("dongbw");
        batchInquiry.setBi_recorduu(userUU);
        batchInquiry.setBi_remark("111");
        List<BatchInProduct> inProducts = new ArrayList<>();

        BatchInProduct inProduct = new BatchInProduct();
        inProduct.setBip_biid(1L);
        inProduct.setBip_currency("RMB");
        inProduct.setBip_detno((short) 1);
        inProduct.setBip_id(1L);
        inProduct.setBip_mobile("15671616315");
        inProduct.setBip_prodcode("PROD001");
        inProduct.setPr_brand("Rohm");
        inProduct.setPr_detail("名称");
        inProduct.setPr_spec("规格");
        inProduct.setYcgqty(1000d);
        inProducts.add(inProduct);
        batchInquiry.setInProducts(inProducts);
        inquiries.add(batchInquiry);

        params.put("inquiries", inquiries);
        params.put("enuu", enUU);
        String res = HttpUtil.doPost(localhost+ "/inquiry/public/check", FlexJsonUtils.toJsonDeep(params));
        System.out.println(JSON.toJSON(res));
    }

    /**
     * /quotation
     */
    @Test
    public void testGetReply() throws Exception {
        HashMap<String, Object> params = new HashMap<>();
        params.put("enuu", enUU);
        HttpUtil.Response res = HttpUtil.sendGetRequest(localhost + "/inquiry/public/quotation", params);
        System.out.println(FlexJsonUtils.toJson(res));
    }

    /**
     * /reply/back
     */
    @Test
    public void testOnReplySuccess() throws Exception {
        HashMap<String, Object> params = new HashMap<>();
        params.put("enuu", enUU);
        params.put("data", "1223,1224,1225,1226,1227");
        HttpUtil.Response res = HttpUtil.sendGetRequest(localhost + "/inquiry/public/quotation", params);
        System.out.println(FlexJsonUtils.toJson(res));
    }

    /**
     * /reply/decide
     * @throws Exception
     */
    @Test
    public void testOnReplyDecide() throws Exception {
        HashMap<String, Object> params = new HashMap<>();
        List<InquiryDecide> inquiryDecides = new ArrayList<>();
        InquiryDecide inquiryDecide = new InquiryDecide();
        inquiryDecide.setB2b_id(1223L);
        inquiryDecide.setB2b_qu_id(122L);
        inquiryDecide.setId_agreed((short) 1);
        inquiryDecide.setId_detno((short) 1);
        inquiryDecide.setIn_code("MALL201809270001");
        inquiryDecides.add(inquiryDecide);
        params.put("decides", inquiryDecides);
        params.put("enuu", enUU);
        String res = HttpUtil.doPost(localhost+ "/inquiry/public/reply/decide", FlexJsonUtils.toJsonDeep(params));
        System.out.println(JSON.toJSON(res));
    }

    /**
     * /invalid
     * @throws Exception
     */
    @Test
    public void testOnReplyInvalid() throws Exception {
        HashMap<String, Object> params = new HashMap<>();
        List<Inquiry> inquiries = new ArrayList<>();
        Inquiry inquiry = new Inquiry();
        inquiry.setIn_code("IN20180929001");
        inquiries.add(inquiry);
        params.put("inquiries", inquiries);
        params.put("enuu", enUU);
        String res = HttpUtil.doPost(localhost+ "/inquiry/public/invalid", FlexJsonUtils.toJsonDeep(params));
        System.out.println(JSON.toJSON(res));
    }

    /**
     * /down/decide
     */
    @Test
    public void testDownloadDecide() throws Exception {
        HashMap<String, Object> params = new HashMap<>();
        params.put("enuu", enUU);
        HttpUtil.Response res = HttpUtil.sendGetRequest(localhost + "/inquiry/public/down/decide", params);
        System.out.println(FlexJsonUtils.toJson(res));
    }

    /**
     * /down/decide/back
     */
    @Test
    public void testDownloadDecideBack() throws Exception {
        HashMap<String, Object> params = new HashMap<>();
        params.put("enuu", enUU);
        params.put("data", "1223,1224,1225,1226,1227");
        HttpUtil.Response res = HttpUtil.sendGetRequest(localhost + "/inquiry/public/down/decide/back", params);
        System.out.println(FlexJsonUtils.toJson(res));
    }

    /**
     * /down/reply
     */
    @Test
    public void testDownReply() throws Exception {
        HashMap<String, Object> params = new HashMap<>();
        params.put("enuu", enUU);
        HttpUtil.Response res = HttpUtil.sendGetRequest(localhost + "/inquiry/public/down/reply", params);
        System.out.println(FlexJsonUtils.toJson(res));
    }

    /**
     * /down/reply/back
     */
    @Test
    public void testOnReplyDownSuccess() throws Exception {
        HashMap<String, Object> params = new HashMap<>();
        params.put("enUU", enUU);
        params.put("ids", "1223,1224,1225,1226,1227");
        HttpUtil.Response res = HttpUtil.sendGetRequest(localhost + "/inquiry/public/down/reply/back", params);
        System.out.println(FlexJsonUtils.toJson(res));
    }

    /**
     * /enremind
     */
    @Test
    public void testDownEnRemind() throws Exception {
        HashMap<String, Object> params = new HashMap<>();
        params.put("vendUU", enUU);
        HttpUtil.Response res = HttpUtil.sendGetRequest(localhost + "/inquiry/public/enremind", params);
        System.out.println(FlexJsonUtils.toJson(res));
    }

    /**
     * /enremind/back
     */
    @Test
    public void testDownEnRemindSuccess() throws Exception {
        HashMap<String, Object> params = new HashMap<>();
        params.put("vendUU", enUU);
        params.put("ids", "1223,1224,1225,1226,1227");
        HttpUtil.Response res = HttpUtil.sendGetRequest(localhost + "/inquiry/public/enremind/back", params);
        System.out.println(FlexJsonUtils.toJson(res));
    }
}
