package com.uas.ps.inquiry.controller;

import com.alibaba.fastjson.JSON;
import com.uas.ps.inquiry.BaseJunitTest;
import com.uas.ps.inquiry.entity.Constant;
import com.uas.ps.inquiry.model.PurcInquiry;
import com.uas.ps.inquiry.model.PurcInquiryItem;
import com.uas.ps.inquiry.page.PageInfo;
import com.uas.ps.inquiry.page.SearchFilter;
import com.uas.ps.inquiry.util.FlexJsonUtils;
import com.uas.ps.inquiry.util.HttpUtil;
import org.junit.Assert;
import org.junit.Test;

import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;

/**
 * @author liuam
 * @since 2018/9/12 0012 上午 11:24
 */

public class InquiryForBuyerControllerTest extends BaseJunitTest {


    /**
     * localhost
     */
    private static final String localhost = "http://10.1.51.82:24002";

    /**
     * dev
     */
    private static final String dev_url = "http://192.168.253.12:24000";

    /**
     * userUU
     */
    private static final Long userUU = 1000002503L;

    /**
     * enUU
     */
    private static final Long enUU = 10041166L;

    /**
     * /detail
     */
    @Test
    public void testFindById() throws Exception {
        Long id = 59L;
        Long enuu = 10041166L;
        HashMap<String, Object> params = new HashMap<>();
        params.put("id", id);
        params.put("enuu", enuu);
        HttpUtil.Response res = HttpUtil.sendGetRequest(localhost + "/inquiry/buyer/save", params);
        Assert.assertEquals(enuu, JSON.parseObject(res.getResponseText(), PurcInquiry.class).getEnUU());
        System.out.println(FlexJsonUtils.toJson(res));
    }

    /**
     * /inquiryList
     */
    @Test
    public void testGetInquiry() throws Exception {
        SearchFilter searchFilter = new SearchFilter();
        searchFilter.setEnUU(1001166L);
        searchFilter.setUserUU(1000002503L);
        PageInfo pageInfo = new PageInfo(1, 20);
        HashMap<String, Object> params = new HashMap<>();
        params.put("pageInfo", pageInfo);
        params.put("searchFilter", searchFilter);
        HttpUtil.Response res = HttpUtil.sendGetRequest(localhost + "/inquiry/buyer/inquiryList", params);
        System.out.println(FlexJsonUtils.toJson(res));
    }

    /**
     * /list
     */
    @Test
    public void testGetInquiryList() throws Exception {
        SearchFilter searchFilter = new SearchFilter();
        searchFilter.setEnUU(1001166L);
        searchFilter.setUserUU(1000002503L);
        PageInfo pageInfo = new PageInfo(1, 20);
        HashMap<String, Object> params = new HashMap<>();
        params.put("pageInfo", pageInfo);
        params.put("searchFilter", searchFilter);
        params.put("state", "done");
        HttpUtil.Response res = HttpUtil.sendGetRequest(localhost + "/inquiry/buyer/list", params);
        System.out.println(FlexJsonUtils.toJson(res));
    }

    /**
     * /product/detail
     */
    @Test
    public void testFindInquiryDetailById() throws Exception {
        Long id = 9L;
        Long enuu = 10041166L;
        HashMap<String, Object> params = new HashMap<>();
        params.put("id", id);
        params.put("enuu", enuu);
        HttpUtil.Response res = HttpUtil.sendGetRequest(localhost + "/inquiry/buyer/list", params);
        System.out.println(FlexJsonUtils.toJson(res));
    }

    /**
     * /quotations
     */
    @Test
    public void testFindQuotationsByPage() throws Exception {
        SearchFilter searchFilter = new SearchFilter();
        searchFilter.setEnUU(1001166L);
        searchFilter.setUserUU(1000002503L);
        PageInfo pageInfo = new PageInfo(1, 20);
        HashMap<String, Object> params = new HashMap<>();
        params.put("pageInfo", pageInfo);
        params.put("searchFilter", searchFilter);
        params.put("enUU", enUU);
        params.put("userUU", userUU);
        HttpUtil.Response res = HttpUtil.sendGetRequest(localhost + "/inquiry/buyer/quotations", params);
        System.out.println(FlexJsonUtils.toJson(res));
    }

    /**
     * /quotation
     */
    @Test
    public void testFindQuotationById() throws Exception {
        HashMap<String, Object> params = new HashMap<>();
        params.put("id", 9L);
        HttpUtil.Response res = HttpUtil.sendGetRequest(localhost + "/inquiry/buyer/quotation", params);
        System.out.println(FlexJsonUtils.toJson(res));
    }

    /**
     * /save
     */
    @Test
    public void testSaveInquiry() throws Exception {
        PurcInquiry inquiry = new PurcInquiry();
        inquiry.setEndDate(new Date(System.currentTimeMillis() + 1000 * 60 * 60 * 24 * 2));
        inquiry.setRecorderUU(1000002503L);
        inquiry.setAmount(1);
        inquiry.setRecorder("董必伟");
        inquiry.setEnUU(10041559L);
        inquiry.setSourceapp("MALL");
        PurcInquiryItem item = new PurcInquiryItem();
        item.setCmpCode("HCD89045");
        item.setDate(new Date(System.currentTimeMillis()));
        item.setUserUU(1000002503L);
        item.setProdTitle("测试15");
        item.setEndDate(new Date(System.currentTimeMillis() + 1000 * 60 * 60 * 24 * 2));
        item.setSource("MALL");
        item.setSpec("14211");
        item.setUserTel("15671616315");
        item.setUserName("董必伟");
        item.setInbrand("A123 Systems");
        Set<PurcInquiryItem> items = new HashSet<>();
        items.add(item);
        inquiry.setInquiryItems(items);
        String res = HttpUtil.doPost("http://10.1.51.82:24002" + "/inquiry/buyer/save",
                FlexJsonUtils.toJsonDeep(inquiry));
        System.out.println(res);
    }

    /**
     * /decide
     */
    @Test
    public void testDecideQuote() throws Exception {
        HashMap<String, Object> params = new HashMap<>();
        params.put("id", 9L);
        params.put("status", Constant.YES);
        params.put("useruu", userUU);
        params.put("enuu", enUU);
        HttpUtil.Response res = HttpUtil.sendGetRequest(localhost + "/inquiry/buyer/decide", params);
        System.out.println(FlexJsonUtils.toJson(res));
    }

    /**
     * /adopt
     */
    @Test
    public void testAdoptQuote() throws Exception {
        HashMap<String, Object> params = new HashMap<>();
        params.put("id", 9L);
        params.put("status", Constant.YES);
        params.put("useruu", userUU);
        params.put("enuu", enUU);
        HttpUtil.Response res = HttpUtil.sendGetRequest(localhost + "/inquiry/buyer/adopt", params);
        System.out.println(FlexJsonUtils.toJson(res));
    }

    /**
     * /refuse
     */
    @Test
    public void testRefuseQuote() throws Exception {
        HashMap<String, Object> params = new HashMap<>();
        params.put("id", 9L);
        params.put("status", Constant.NO);
        params.put("refusereason", "拒绝理由1");
        params.put("useruu", userUU);
        params.put("enuu", enUU);
        HttpUtil.Response res = HttpUtil.sendGetRequest(localhost + "/inquiry/buyer/refuse", params);
        System.out.println(FlexJsonUtils.toJson(res));
    }

}
